;;; php-mode.el --- major mode for editing PHP code

;; Copyright (C) 1999-2004 Turadg Aleahmad

;; Maintainer: Turadg Aleahmad <turadg at users.sourceforge.net>
;; Keywords: php languages oop
;; Created: 1999-05-17
;; Modified: 2004-01-24
;; X-URL:   http://php-mode.sourceforge.net/

(defconst php-version "1.1.0"
  "PHP Mode version number.")

;;; License

;; This program is free software; you can redistribute it and/or
;; modify it under the terms of the GNU General Public License
;; as published by the Free Software Foundation; either version 2
;; of the License, or (at your option) any later version.

;; This program is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with this program; if not, write to the Free Software
;; Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

;;; Usage

;; Rename this file to php-mode.el if it isn't already then place it in
;; your Emacs lisp path (eg. site-lisp) and add to your .emacs file:
;;   (require 'php-mode)

;; If you want colorization, turn on global-font-lock or
;; add this to your .emacs:
;;   (add-hook 'php-mode-user-hook 'turn-on-font-lock)

;; To use abbrev-mode, add lines like this:
;;   (add-hook 'php-mode-user-hook
;;     '(lambda () (define-abbrev php-mode-abbrev-table "ex" "extends")))

;; To make php-mode compatible with html-mode, see http://php-mode.sf.net

;; Many options available under Help:Customize
;; Options specific to php-mode are in
;;  Programming/Languages/Php
;; Since it inherits much functionality from c-mode, look there too
;;  Programming/Languages/C

;;; Commentary:

;; PHP mode is a major mode for editing PHP 3 and 4 source code.  It's
;; an extension of C mode; thus it inherits all C mode's navigation
;; functionality.  But it colors according to the PHP grammar and indents
;; according to the PEAR coding guidelines.  It also includes a couple
;; handy IDE-type features such as documentation search and a source
;; and class browser.


;;; Contributors: (in chronological order)

;; Juanjo, Torsten Martinsen, Vinai Kopp, Sean Champ, Doug Marcey,
;; Kevin Blake, Rex McMaster, Mathias Meyer, Boris Folgmann, Roland
;; Rosenfeld, Fred Yankowski, Craig Andrews, John Keller, Ryan
;; Sammartino, ppercot, Valentin Funk, Stig Bakken, Gregory Stark,
;; Chris Morris, Nils Rennebarth, Gerrit Riessen, Eric Mc Sween,
;; Ville Skytta, Giacomo Tesio

;;; Changelog:

;; 1.1
;;   Added PHP5 support (Giacomo Tesio)
;;     known problem: doesn't highlight after first 'implements'
;;   Better XEmacs compatibility (imenu, regexp, and comments!) (Ville Skytta)
;;   Improvement to php-conditional-key regexp (Eric Mc Sween)

;; 1.05
;;   Incorporated speedbar defs by Gerrit Riessen
;;   Add "foreach" to conditional introducing keywords (Nils Rennebarth)
;;   Cleared the Changelog
;;   Moved contribution credits into comments above


;;; Code:

(require 'speedbar)
(require 'font-lock)
(require 'cc-mode)
(require 'custom)
(require 'etags)
(eval-when-compile
  (require 'regexp-opt))
(require 'apropos)
(require 'cl)
(require 'c-comment-edit)

;; Local variables
(defgroup php nil
  "Major mode for editing PHP code."
  :prefix "php-"
  :group 'languages)

(defcustom php-speedbar-config t
  "*When set to true automatically configures Speedbar to observe PHP files.\
Ignores php-file patterns option; fixed to expression \"\\.\\(inc\\|php[s34]?\\)\""
  :type 'boolean
  :group 'php)

(defcustom php-mode-speedbar-open nil
  "Normally php-mode starts with the speedbar closed.\
Turning this on will open it whenever php-mode is loaded."
  :type 'boolean
  :group 'php)

(defcustom php-manual-url "http://www.php.net/manual/en/manual.php"
  "*URL at which to find PHP manual.\
You can replace \"en\" with your ISO language code."
  :type 'string
  :group 'php)

(defcustom php-search-url "http://www.php.net/"
  "*URL at which to search for documentation on a word"
  :type 'string
  :group 'php)

;;;###autoload
(defcustom php-file-patterns (list "\\.php[s34]?\\'" "\\.phtml\\'" "\\.inc\\'")
  "*List of file patterns for which to automatically invoke php-mode."
  :type '(repeat (regexp :tag "Pattern"))
  :group 'php)

(defcustom php-mode-user-hook nil
  "List of functions to be executed on entry to php-mode"
  :type 'hook
  :group 'php)

(defcustom php-mode-force-pear nil
  "Normally PEAR coding rules are enforced only when the filename contains \"PEAR\"\
Turning this on will force PEAR rules on all PHP files."
  :type 'boolean
  :group 'php)

;; Note whether we're in XEmacs
(defconst xemacsp (string-match "Lucid\\|XEmacs" emacs-version)
  "Non nil if using XEmacs.")

;;; Typing shortcuts
(define-skeleton php-insert-comment
  "Inserts a full comment." nil
  "/* " _ " */")

;; Define the imenu-generic-expression for PHP mode.
;; To use, execute M-x imenu, then click on Functions or Classes,
;; then select given function/class name to go to its definition.
;; [Contributed by Gerrit Riessen]
(defvar php-imenu-generic-expression
 '(
   ("Functions"
    "^\\s-*function\\s-+\\([a-zA-Z0-9_]+\\)\\s-*(" 1)
   ("Classes"
    "^\\s-*class\\s-+\\([a-zA-Z0-9_]+\\)\\s-*" 1)
   )
 "Imenu generic expression for PHP Mode. See `imenu-generic-expression'."
 )

;; Add "foreach" to conditional introducing keywords
(defvar php-conditional-key nil)
(let ((all-kws "for\\|if\\|do\\|else\\|while\\|switch\\|foreach\\|elseif\\|try\\|finally\\|try\\|catch all")
      (front   "\\<\\(")
      (back    "\\)\\>[^_]"))
  (setq php-conditional-key (concat front all-kws back)))

(defconst php-class-kwds "class\\|interface")

(defconst php-class-key
  (concat
   "\\(" php-class-kwds "\\)\\s +"
   c-symbol-key                   ;name of the class
   "\\(\\s *extends\\s *" c-symbol-key "\\)?" ;maybe followed by superclass
   "\\(\\s *implements *[^{]+{\\)?"       ;maybe the adopted protocols list
   ))

;;;###autoload
(define-derived-mode php-mode c-mode "PHP"
  "Major mode for editing PHP code.\n\n\\{php-mode-map}"

  (setq comment-start "/* "
    comment-end   " */"
    comment-start-skip "// *")

  (setq c-class-key php-class-key)
  (setq c-conditional-key php-conditional-key)

  (defvar php-mode-syntax-table php-mode-syntax-table)
  ;; this line makes $ into punctuation instead of a word constituent
  ;; it used to be active, but it killed indenting of case lines that
  ;; begin with '$' (many do).  If anyone has a solution to this
  ;; problem, please let me know.  Of course, you're welcome to
  ;; uncomment this line in your installation.
;  (modify-syntax-entry ?$ "." php-mode-syntax-table)

  ;; The above causes XEmacs to handle shell-style comments correctly,
  ;; but fails to work in GNU Emacs which fails to interpret \n as the
  ;; end of the comment.
  (if xemacsp (progn
        (modify-syntax-entry ?# "< b" php-mode-syntax-table)
        (modify-syntax-entry ?\n "> b" php-mode-syntax-table)
        ))

  (make-local-variable 'font-lock-defaults)
  (setq font-lock-defaults
    '((php-font-lock-keywords-1
       php-font-lock-keywords-2
       ;; Comment-out the next line if the font-coloring is too
       ;; extreme/ugly for you.
       php-font-lock-keywords-3
       )
      nil               ; KEYWORDS-ONLY
      T             ; CASE-FOLD
      nil               ; SYNTAX-ALIST
      nil               ; SYNTAX-BEGIN
      (font-lock-syntactic-keywords . php-font-lock-syntactic-keywords)))

  (setq font-lock-maximum-decoration t
    case-fold-search t      ; PHP vars are case-sensitive
    imenu-generic-expression php-imenu-generic-expression)

  ;; Do not force newline at end of file.  Such newlines can cause
  ;; trouble if the PHP file is included in another file before calls
  ;; to header() or cookie().
  (set (make-local-variable 'require-final-newline) nil)
  (set (make-local-variable 'next-line-add-newlines) nil)

  ;; PEAR coding standards
;;  (make-local-hook 'php-mode-pear-hook) ;; 21.1Τ make-local-hookɬ
  (add-hook 'php-mode-pear-hook
        (lambda nil (set (make-local-variable 'tab-width) 4)) nil t)
  (add-hook 'php-mode-pear-hook
        (lambda nil (set (make-local-variable 'c-basic-offset) 4)) nil t)
  (add-hook 'php-mode-pear-hook
        (lambda nil (set (make-local-variable 'c-hanging-comment-ender-p) nil)) nil t)
  (add-hook 'php-mode-pear-hook
        (lambda nil (set (make-local-variable 'indent-tabs-mode) nil)) nil t)
  (add-hook 'php-mode-pear-hook
        (lambda nil (c-set-offset 'block-open' - )) nil t)
  (add-hook 'php-mode-pear-hook
        (lambda nil (c-set-offset 'block-close' 0 )) nil t)

  (if (or php-mode-force-pear
      (and (stringp buffer-file-name)
           (string-match "PEAR\\|pear"
                 (buffer-file-name))
           (string-match "\\.php$" (buffer-file-name))))
      (run-hooks 'php-mode-pear-hook))

  (run-hooks 'php-mode-user-hook)
  )

;; Make php-mode the default mode for PHP source code buffers.
;;;###autoload
(let ((php-file-patterns-temp php-file-patterns))
  (while php-file-patterns-temp
    (add-to-list 'auto-mode-alist
          (cons (car php-file-patterns-temp) 'php-mode))
    (setq php-file-patterns-temp (cdr php-file-patterns-temp))))

;; Handle Speedbar
(if php-mode-speedbar-open
    (speedbar 1))
(if (and php-speedbar-config (symbolp 'speedbar))
    (speedbar-add-supported-extension "\\.\\(p?inc\\|php[s34]?\\|phtml\\|p[34]\\)"))

;; Make a menu keymap (with a prompt string)
;; and make it the menu bar item's definition.
(define-key php-mode-map "\C-c\C-d" 'php-search-documentation)
(define-key php-mode-map "\C-c\C-m" 'php-browse-manual)
(define-key php-mode-map "\C-c\C-w" 'php-insert-comment)
(define-key php-mode-map "\C-c\C-e" 'c-comment-edit)
(define-key php-mode-map "\M-\t" 'php-complete-function)
(define-key php-mode-map "\C-c\C-s" 'php-complete-superglobal)
(define-key php-mode-map "\C-c\C-c" 'php-complete-constant)
(define-key php-mode-map "\C-c\C-k" 'php-complete-keyword)
(define-key php-mode-map "\C-c\C-t" 'php-complete-type)
(define-key php-mode-map "\C-x." 'php-show-arglist)
(define-key php-mode-map [menu-bar] (make-sparse-keymap))
(define-key php-mode-map [menu-bar php]
  (cons "PHP" (make-sparse-keymap "PHP")))

;; Define specific subcommands in this menu.
(define-key php-mode-map [menu-bar php complete-function]
  '("Complete function name" . php-complete-function))
(define-key php-mode-map
  [menu-bar php complete-keyword]
  '("Complete keyword name" . php-complete-keyword))
(define-key php-mode-map
  [menu-bar php complete-superglobal]
  '("Complete superglobal name" . php-complete-superglobal))
(define-key php-mode-map
  [menu-bar php complete-type]
  '("Complete type name" . php-complete-type))
(define-key php-mode-map
  [menu-bar php complete-constant]
  '("Complete constant name" . php-complete-constant))
(define-key php-mode-map
  [menu-bar php browse-manual]
  '("Browse manual" . php-browse-manual))
(define-key php-mode-map
  [menu-bar php search-documentation]
  '("Search documentation" . php-search-documentation))

;; Define function documentation function
(defun php-search-documentation ()
  "Search PHP documentation for the word at the point."
  (interactive)
  (browse-url (concat php-search-url (current-word t)))
  )

;; Define function for browsing manual
(defun php-browse-manual ()
  "Bring up manual for PHP."
  (interactive)
  (browse-url php-manual-url)
  )

;; Find the pattern we want to complete
;; find-tag-default from GNU Emacs etags.el
(defun php-get-pattern ()
  (save-excursion
    (while (looking-at "\\sw\\|\\s_")
      (forward-char 1))
    (if (or (re-search-backward "\\sw\\|\\s_"
                (save-excursion (beginning-of-line) (point))
                t)
        (re-search-forward "\\(\\sw\\|\\s_\\)+"
                   (save-excursion (end-of-line) (point))
                   t))
    (progn (goto-char (match-end 0))
           (buffer-substring-no-properties
                (point)
                (progn (forward-sexp -1)
                       (while (looking-at "\\s'")
                         (forward-char 1))
                       (point))))
      nil)))


(defun php-show-arglist ()
  (interactive)
  (let* ((tagname (php-get-pattern))
         (buf (find-tag-noselect tagname nil nil))
         arglist)
    (save-excursion
      (set-buffer buf)
      (goto-char (point-min))
      (when (re-search-forward
             (format "function[ \t]+%s[ \t]*(\\([^{]*\\))" tagname)
             nil t)
        (setq arglist (buffer-substring-no-properties
                       (match-beginning 1) (match-end 1)))))
    (if arglist
        (message "Arglist for %s: %s" tagname arglist)
        (message "unknown function: %s" tagname))))

;; Define shortcut
(define-key php-mode-map
  "\C-c\C-f"
  'php-search-documentation)

;; Define shortcut
(define-key php-mode-map
  "\C-c\C-m"
  'php-browse-manual)

(defconst php-constants
     '(;; core constants
       "__LINE__" "__FILE__"
       "PHP_OS" "PHP_VERSION"
       "TRUE" "FALSE" "NULL"
       "E_ERROR" "E_NOTICE" "E_PARSE" "E_WARNING" "E_ALL"
       "E_USER_ERROR" "E_USER_WARNING" "E_USER_NOTICE"
       "DEFAULT_INCLUDE_PATH" "PEAR_INSTALL_DIR" "PEAR_EXTENSION_DIR"
       "PHP_BINDIR" "PHP_LIBDIR" "PHP_DATADIR" "PHP_SYSCONFDIR"
       "PHP_LOCALSTATEDIR" "PHP_CONFIG_FILE_PATH"
       ;; from ext/standard:
       "EXTR_OVERWRITE" "EXTR_SKIP" "EXTR_PREFIX_SAME"
       "EXTR_PREFIX_ALL" "EXTR_PREFIX_INVALID" "SORT_ASC" "SORT_DESC"
       "SORT_REGULAR" "SORT_NUMERIC" "SORT_STRING" "ASSERT_ACTIVE"
       "ASSERT_CALLBACK" "ASSERT_BAIL" "ASSERT_WARNING"
       "ASSERT_QUIET_EVAL" "CONNECTION_ABORTED" "CONNECTION_NORMAL"
       "CONNECTION_TIMEOUT" "M_E" "M_LOG2E" "M_LOG10E" "M_LN2"
       "M_LN10" "M_PI" "M_PI_2" "M_PI_4" "M_1_PI" "M_2_PI"
       "M_2_SQRTPI" "M_SQRT2" "M_SQRT1_2" "CRYPT_SALT_LENGTH"
       "CRYPT_STD_DES" "CRYPT_EXT_DES" "CRYPT_MD5" "CRYPT_BLOWFISH"
       "DIRECTORY_SEPARATOR" "SEEK_SET" "SEEK_CUR" "SEEK_END"
       "LOCK_SH" "LOCK_EX" "LOCK_UN" "LOCK_NB" "HTML_SPECIALCHARS"
       "HTML_ENTITIES" "ENT_COMPAT" "ENT_QUOTES" "ENT_NOQUOTES"
       "INFO_GENERAL" "INFO_CREDITS" "INFO_CONFIGURATION"
       "INFO_ENVIRONMENT" "INFO_VARIABLES" "INFO_LICENSE" "INFO_ALL"
       "CREDITS_GROUP" "CREDITS_GENERAL" "CREDITS_SAPI"
       "CREDITS_MODULES" "CREDITS_DOCS" "CREDITS_FULLPAGE"
       "CREDITS_QA" "CREDITS_ALL" "PHP_OUTPUT_HANDLER_START"
       "PHP_OUTPUT_HANDLER_CONT" "PHP_OUTPUT_HANDLER_END"
       "STR_PAD_LEFT" "STR_PAD_RIGHT" "STR_PAD_BOTH"
       "PATHINFO_DIRNAME" "PATHINFO_BASENAME" "PATHINFO_EXTENSION"
       "CHAR_MAX" "LC_CTYPE" "LC_NUMERIC" "LC_TIME" "LC_COLLATE"
       "LC_MONETARY" "LC_ALL" "LC_MESSAGES" "LOG_EMERG" "LOG_ALERT"
       "LOG_CRIT" "LOG_ERR" "LOG_WARNING" "LOG_NOTICE" "LOG_INFO"
       "LOG_DEBUG" "LOG_KERN" "LOG_USER" "LOG_MAIL" "LOG_DAEMON"
       "LOG_AUTH" "LOG_SYSLOG" "LOG_LPR" "LOG_NEWS" "LOG_UUCP"
       "LOG_CRON" "LOG_AUTHPRIV" "LOG_LOCAL0" "LOG_LOCAL1"
       "LOG_LOCAL2" "LOG_LOCAL3" "LOG_LOCAL4" "LOG_LOCAL5"
       "LOG_LOCAL6" "LOG_LOCAL7" "LOG_PID" "LOG_CONS" "LOG_ODELAY"
       "LOG_NDELAY" "LOG_NOWAIT" "LOG_PERROR"

       ;; Disabled by default because they slow buffer loading
       ;; If you have use for them, decomment the strings
       ;; that you want colored.
       ;; To compile, you may have to increase 'max-specpdl-size'

       ;; from other bundled extensions:
;        "CAL_EASTER_TO_xxx" "VT_NULL" "VT_EMPTY" "VT_UI1" "VT_I2"
;        "VT_I4" "VT_R4" "VT_R8" "VT_BOOL" "VT_ERROR" "VT_CY" "VT_DATE"
;        "VT_BSTR" "VT_DECIMAL" "VT_UNKNOWN" "VT_DISPATCH" "VT_VARIANT"
;        "VT_I1" "VT_UI2" "VT_UI4" "VT_INT" "VT_UINT" "VT_ARRAY"
;        "VT_BYREF" "CP_ACP" "CP_MACCP" "CP_OEMCP" "CP_SYMBOL"
;        "CP_THREAD_ACP" "CP_UTF7" "CP_UTF8" "CPDF_PM_NONE"
;        "CPDF_PM_OUTLINES" "CPDF_PM_THUMBS" "CPDF_PM_FULLSCREEN"
;        "CPDF_PL_SINGLE" "CPDF_PL_1COLUMN" "CPDF_PL_2LCOLUMN"
;        "CPDF_PL_2RCOLUMN" "CURLOPT_PORT" "CURLOPT_FILE"
;        "CURLOPT_INFILE" "CURLOPT_INFILESIZE" "CURLOPT_URL"
;        "CURLOPT_PROXY" "CURLOPT_VERBOSE" "CURLOPT_HEADER"
;        "CURLOPT_HTTPHEADER" "CURLOPT_NOPROGRESS" "CURLOPT_NOBODY"
;        "CURLOPT_FAILONERROR" "CURLOPT_UPLOAD" "CURLOPT_POST"
;        "CURLOPT_FTPLISTONLY" "CURLOPT_FTPAPPEND" "CURLOPT_NETRC"
;        "CURLOPT_FOLLOWLOCATION" "CURLOPT_FTPASCII" "CURLOPT_PUT"
;        "CURLOPT_MUTE" "CURLOPT_USERPWD" "CURLOPT_PROXYUSERPWD"
;        "CURLOPT_RANGE" "CURLOPT_TIMEOUT" "CURLOPT_POSTFIELDS"
;        "CURLOPT_REFERER" "CURLOPT_USERAGENT" "CURLOPT_FTPPORT"
;        "CURLOPT_LOW_SPEED_LIMIT" "CURLOPT_LOW_SPEED_TIME"
;        "CURLOPT_RESUME_FROM" "CURLOPT_COOKIE" "CURLOPT_SSLCERT"
;        "CURLOPT_SSLCERTPASSWD" "CURLOPT_WRITEHEADER"
;        "CURLOPT_COOKIEFILE" "CURLOPT_SSLVERSION"
;        "CURLOPT_TIMECONDITION" "CURLOPT_TIMEVALUE"
;        "CURLOPT_CUSTOMREQUEST" "CURLOPT_STDERR" "CURLOPT_TRANSFERTEXT"
;        "CURLOPT_RETURNTRANSFER" "CURLOPT_QUOTE" "CURLOPT_POSTQUOTE"
;        "CURLOPT_INTERFACE" "CURLOPT_KRB4LEVEL"
;        "CURLOPT_HTTPPROXYTUNNEL" "CURLOPT_FILETIME"
;        "CURLOPT_WRITEFUNCTION" "CURLOPT_READFUNCTION"
;        "CURLOPT_PASSWDFUNCTION" "CURLOPT_HEADERFUNCTION"
;        "CURLOPT_MAXREDIRS" "CURLOPT_MAXCONNECTS" "CURLOPT_CLOSEPOLICY"
;        "CURLOPT_FRESH_CONNECT" "CURLOPT_FORBID_REUSE"
;        "CURLOPT_RANDOM_FILE" "CURLOPT_EGDSOCKET"
;        "CURLOPT_CONNECTTIMEOUT" "CURLOPT_SSL_VERIFYPEER"
;        "CURLOPT_CAINFO" "CURLOPT_BINARYTRANSER"
;        "CURLCLOSEPOLICY_LEAST_RECENTLY_USED" "CURLCLOSEPOLICY_OLDEST"
;        "CURLINFO_EFFECTIVE_URL" "CURLINFO_HTTP_CODE"
;        "CURLINFO_HEADER_SIZE" "CURLINFO_REQUEST_SIZE"
;        "CURLINFO_TOTAL_TIME" "CURLINFO_NAMELOOKUP_TIME"
;        "CURLINFO_CONNECT_TIME" "CURLINFO_PRETRANSFER_TIME"
;        "CURLINFO_SIZE_UPLOAD" "CURLINFO_SIZE_DOWNLOAD"
;        "CURLINFO_SPEED_DOWNLOAD" "CURLINFO_SPEED_UPLOAD"
;        "CURLINFO_FILETIME" "CURLE_OK" "CURLE_UNSUPPORTED_PROTOCOL"
;        "CURLE_FAILED_INIT" "CURLE_URL_MALFORMAT"
;        "CURLE_URL_MALFORMAT_USER" "CURLE_COULDNT_RESOLVE_PROXY"
;        "CURLE_COULDNT_RESOLVE_HOST" "CURLE_COULDNT_CONNECT"
;        "CURLE_FTP_WEIRD_SERVER_REPLY" "CURLE_FTP_ACCESS_DENIED"
;        "CURLE_FTP_USER_PASSWORD_INCORRECT"
;        "CURLE_FTP_WEIRD_PASS_REPLY" "CURLE_FTP_WEIRD_USER_REPLY"
;        "CURLE_FTP_WEIRD_PASV_REPLY" "CURLE_FTP_WEIRD_227_FORMAT"
;        "CURLE_FTP_CANT_GET_HOST" "CURLE_FTP_CANT_RECONNECT"
;        "CURLE_FTP_COULDNT_SET_BINARY" "CURLE_PARTIAL_FILE"
;        "CURLE_FTP_COULDNT_RETR_FILE" "CURLE_FTP_WRITE_ERROR"
;        "CURLE_FTP_QUOTE_ERROR" "CURLE_HTTP_NOT_FOUND"
;        "CURLE_WRITE_ERROR" "CURLE_MALFORMAT_USER"
;        "CURLE_FTP_COULDNT_STOR_FILE" "CURLE_READ_ERROR"
;        "CURLE_OUT_OF_MEMORY" "CURLE_OPERATION_TIMEOUTED"
;        "CURLE_FTP_COULDNT_SET_ASCII" "CURLE_FTP_PORT_FAILED"
;        "CURLE_FTP_COULDNT_USE_REST" "CURLE_FTP_COULDNT_GET_SIZE"
;        "CURLE_HTTP_RANGE_ERROR" "CURLE_HTTP_POST_ERROR"
;        "CURLE_SSL_CONNECT_ERROR" "CURLE_FTP_BAD_DOWNLOAD_RESUME"
;        "CURLE_FILE_COULDNT_READ_FILE" "CURLE_LDAP_CANNOT_BIND"
;        "CURLE_LDAP_SEARCH_FAILED" "CURLE_LIBRARY_NOT_FOUND"
;        "CURLE_FUNCTION_NOT_FOUND" "CURLE_ABORTED_BY_CALLBACK"
;        "CURLE_BAD_FUNCTION_ARGUMENT" "CURLE_BAD_CALLING_ORDER"
;        "CURLE_HTTP_PORT_FAILED" "CURLE_BAD_PASSWORD_ENTERED"
;        "CURLE_TOO_MANY_REDIRECTS" "CURLE_UNKOWN_TELNET_OPTION"
;        "CURLE_TELNET_OPTION_SYNTAX" "CURLE_ALREADY_COMPLETE"
;        "DBX_MYSQL" "DBX_ODBC" "DBX_PGSQL" "DBX_MSSQL" "DBX_PERSISTENT"
;        "DBX_RESULT_INFO" "DBX_RESULT_INDEX" "DBX_RESULT_ASSOC"
;        "DBX_CMP_TEXT" "DBX_CMP_NUMBER" "XML_ELEMENT_NODE"
;        "XML_ATTRIBUTE_NODE" "XML_TEXT_NODE" "XML_CDATA_SECTION_NODE"
;        "XML_ENTITY_REF_NODE" "XML_ENTITY_NODE" "XML_PI_NODE"
;        "XML_COMMENT_NODE" "XML_DOCUMENT_NODE" "XML_DOCUMENT_TYPE_NODE"
;        "XML_DOCUMENT_FRAG_NODE" "XML_NOTATION_NODE"
;        "XML_HTML_DOCUMENT_NODE" "XML_DTD_NODE" "XML_ELEMENT_DECL_NODE"
;        "XML_ATTRIBUTE_DECL_NODE" "XML_ENTITY_DECL_NODE"
;        "XML_NAMESPACE_DECL_NODE" "XML_GLOBAL_NAMESPACE"
;        "XML_LOCAL_NAMESPACE" "XML_ATTRIBUTE_CDATA" "XML_ATTRIBUTE_ID"
;        "XML_ATTRIBUTE_IDREF" "XML_ATTRIBUTE_IDREFS"
;        "XML_ATTRIBUTE_ENTITY" "XML_ATTRIBUTE_NMTOKEN"
;        "XML_ATTRIBUTE_NMTOKENS" "XML_ATTRIBUTE_ENUMERATION"
;        "XML_ATTRIBUTE_NOTATION" "XPATH_UNDEFINED" "XPATH_NODESET"
;        "XPATH_BOOLEAN" "XPATH_NUMBER" "XPATH_STRING" "XPATH_POINT"
;        "XPATH_RANGE" "XPATH_LOCATIONSET" "XPATH_USERS" "FBSQL_ASSOC"
;        "FBSQL_NUM" "FBSQL_BOTH" "FDFValue" "FDFStatus" "FDFFile"
;        "FDFID" "FDFFf" "FDFSetFf" "FDFClearFf" "FDFFlags" "FDFSetF"
;        "FDFClrF" "FDFAP" "FDFAS" "FDFAction" "FDFAA" "FDFAPRef"
;        "FDFIF" "FDFEnter" "FDFExit" "FDFDown" "FDFUp" "FDFFormat"
;        "FDFValidate" "FDFKeystroke" "FDFCalculate"
;        "FRIBIDI_CHARSET_UTF8" "FRIBIDI_CHARSET_8859_6"
;        "FRIBIDI_CHARSET_8859_8" "FRIBIDI_CHARSET_CP1255"
;        "FRIBIDI_CHARSET_CP1256" "FRIBIDI_CHARSET_ISIRI_3342"
;        "FTP_ASCII" "FTP_BINARY" "FTP_IMAGE" "FTP_TEXT" "IMG_GIF"
;        "IMG_JPG" "IMG_JPEG" "IMG_PNG" "IMG_WBMP" "IMG_COLOR_TILED"
;        "IMG_COLOR_STYLED" "IMG_COLOR_BRUSHED"
;        "IMG_COLOR_STYLEDBRUSHED" "IMG_COLOR_TRANSPARENT"
;        "IMG_ARC_ROUNDED" "IMG_ARC_PIE" "IMG_ARC_CHORD"
;        "IMG_ARC_NOFILL" "IMG_ARC_EDGED" "GMP_ROUND_ZERO"
;        "GMP_ROUND_PLUSINF" "GMP_ROUND_MINUSINF" "HW_ATTR_LANG"
;        "HW_ATTR_NR" "HW_ATTR_NONE" "IIS_READ" "IIS_WRITE"
;        "IIS_EXECUTE" "IIS_SCRIPT" "IIS_ANONYMOUS" "IIS_BASIC"
;        "IIS_NTLM" "NIL" "OP_DEBUG" "OP_READONLY" "OP_ANONYMOUS"
;        "OP_SHORTCACHE" "OP_SILENT" "OP_PROTOTYPE" "OP_HALFOPEN"
;        "OP_EXPUNGE" "OP_SECURE" "CL_EXPUNGE" "FT_UID" "FT_PEEK"
;        "FT_NOT" "FT_INTERNAL" "FT_PREFETCHTEXT" "ST_UID" "ST_SILENT"
;        "ST_SET" "CP_UID" "CP_MOVE" "SE_UID" "SE_FREE" "SE_NOPREFETCH"
;        "SO_FREE" "SO_NOSERVER" "SA_MESSAGES" "SA_RECENT" "SA_UNSEEN"
;        "SA_UIDNEXT" "SA_UIDVALIDITY" "SA_ALL" "LATT_NOINFERIORS"
;        "LATT_NOSELECT" "LATT_MARKED" "LATT_UNMARKED" "SORTDATE"
;        "SORTARRIVAL" "SORTFROM" "SORTSUBJECT" "SORTTO" "SORTCC"
;        "SORTSIZE" "TYPETEXT" "TYPEMULTIPART" "TYPEMESSAGE"
;        "TYPEAPPLICATION" "TYPEAUDIO" "TYPEIMAGE" "TYPEVIDEO"
;        "TYPEOTHER" "ENC7BIT" "ENC8BIT" "ENCBINARY" "ENCBASE64"
;        "ENCQUOTEDPRINTABLE" "ENCOTHER" "INGRES_ASSOC" "INGRES_NUM"
;        "INGRES_BOTH" "IBASE_DEFAULT" "IBASE_TEXT" "IBASE_UNIXTIME"
;        "IBASE_READ" "IBASE_COMMITTED" "IBASE_CONSISTENCY"
;        "IBASE_NOWAIT" "IBASE_TIMESTAMP" "IBASE_DATE" "IBASE_TIME"
;        "LDAP_DEREF_NEVER" "LDAP_DEREF_SEARCHING" "LDAP_DEREF_FINDING"
;        "LDAP_DEREF_ALWAYS" "LDAP_OPT_DEREF" "LDAP_OPT_SIZELIMIT"
;        "LDAP_OPT_TIMELIMIT" "LDAP_OPT_PROTOCOL_VERSION"
;        "LDAP_OPT_ERROR_NUMBER" "LDAP_OPT_REFERRALS" "LDAP_OPT_RESTART"
;        "LDAP_OPT_HOST_NAME" "LDAP_OPT_ERROR_STRING"
;        "LDAP_OPT_MATCHED_DN" "LDAP_OPT_SERVER_CONTROLS"
;        "LDAP_OPT_CLIENT_CONTROLS" "GSLC_SSL_NO_AUTH"
;        "GSLC_SSL_ONEWAY_AUTH" "GSLC_SSL_TWOWAY_AUTH" "MCAL_SUNDAY"
;        "MCAL_MONDAY" "MCAL_TUESDAY" "MCAL_WEDNESDAY" "MCAL_THURSDAY"
;        "MCAL_FRIDAY" "MCAL_SATURDAY" "MCAL_JANUARY" "MCAL_FEBRUARY"
;        "MCAL_MARCH" "MCAL_APRIL" "MCAL_MAY" "MCAL_JUNE" "MCAL_JULY"
;        "MCAL_AUGUST" "MCAL_SEPTEMBER" "MCAL_OCTOBER" "MCAL_NOVEMBER"
;        "MCAL_RECUR_NONE" "MCAL_RECUR_DAILY" "MCAL_RECUR_WEEKLY"
;        "MCAL_RECUR_MONTHLY_MDAY" "MCAL_RECUR_MONTHLY_WDAY"
;        "MCAL_RECUR_YEARLY" "MCAL_M_SUNDAY" "MCAL_M_MONDAY"
;        "MCAL_M_TUESDAY" "MCAL_M_WEDNESDAY" "MCAL_M_THURSDAY"
;        "MCAL_M_FRIDAY" "MCAL_M_SATURDAY" "MCAL_M_WEEKDAYS"
;        "MCAL_M_WEEKEND" "MCAL_M_ALLDAYS" "MCRYPT_" "MCRYPT_"
;        "MCRYPT_ENCRYPT" "MCRYPT_DECRYPT" "MCRYPT_DEV_RANDOM"
;        "MCRYPT_DEV_URANDOM" "MCRYPT_RAND" "SWFBUTTON_HIT"
;        "SWFBUTTON_DOWN" "SWFBUTTON_OVER" "SWFBUTTON_UP"
;        "SWFBUTTON_MOUSEUPOUTSIDE" "SWFBUTTON_DRAGOVER"
;        "SWFBUTTON_DRAGOUT" "SWFBUTTON_MOUSEUP" "SWFBUTTON_MOUSEDOWN"
;        "SWFBUTTON_MOUSEOUT" "SWFBUTTON_MOUSEOVER"
;        "SWFFILL_RADIAL_GRADIENT" "SWFFILL_LINEAR_GRADIENT"
;        "SWFFILL_TILED_BITMAP" "SWFFILL_CLIPPED_BITMAP"
;        "SWFTEXTFIELD_HASLENGTH" "SWFTEXTFIELD_NOEDIT"
;        "SWFTEXTFIELD_PASSWORD" "SWFTEXTFIELD_MULTILINE"
;        "SWFTEXTFIELD_WORDWRAP" "SWFTEXTFIELD_DRAWBOX"
;        "SWFTEXTFIELD_NOSELECT" "SWFTEXTFIELD_HTML"
;        "SWFTEXTFIELD_ALIGN_LEFT" "SWFTEXTFIELD_ALIGN_RIGHT"
;        "SWFTEXTFIELD_ALIGN_CENTER" "SWFTEXTFIELD_ALIGN_JUSTIFY"
;        "UDM_FIELD_URLID" "UDM_FIELD_URL" "UDM_FIELD_CONTENT"
;        "UDM_FIELD_TITLE" "UDM_FIELD_KEYWORDS" "UDM_FIELD_DESC"
;        "UDM_FIELD_DESCRIPTION" "UDM_FIELD_TEXT" "UDM_FIELD_SIZE"
;        "UDM_FIELD_RATING" "UDM_FIELD_SCORE" "UDM_FIELD_MODIFIED"
;        "UDM_FIELD_ORDER" "UDM_FIELD_CRC" "UDM_FIELD_CATEGORY"
;        "UDM_PARAM_PAGE_SIZE" "UDM_PARAM_PAGE_NUM"
;        "UDM_PARAM_SEARCH_MODE" "UDM_PARAM_CACHE_MODE"
;        "UDM_PARAM_TRACK_MODE" "UDM_PARAM_PHRASE_MODE"
;        "UDM_PARAM_CHARSET" "UDM_PARAM_STOPTABLE"
;        "UDM_PARAM_STOP_TABLE" "UDM_PARAM_STOPFILE"
;        "UDM_PARAM_STOP_FILE" "UDM_PARAM_WEIGHT_FACTOR"
;        "UDM_PARAM_WORD_MATCH" "UDM_PARAM_MAX_WORD_LEN"
;        "UDM_PARAM_MAX_WORDLEN" "UDM_PARAM_MIN_WORD_LEN"
;        "UDM_PARAM_MIN_WORDLEN" "UDM_PARAM_ISPELL_PREFIXES"
;        "UDM_PARAM_ISPELL_PREFIX" "UDM_PARAM_PREFIXES"
;        "UDM_PARAM_PREFIX" "UDM_PARAM_CROSS_WORDS"
;        "UDM_PARAM_CROSSWORDS" "UDM_LIMIT_CAT" "UDM_LIMIT_URL"
;        "UDM_LIMIT_TAG" "UDM_LIMIT_LANG" "UDM_LIMIT_DATE"
;        "UDM_PARAM_FOUND" "UDM_PARAM_NUM_ROWS" "UDM_PARAM_WORDINFO"
;        "UDM_PARAM_WORD_INFO" "UDM_PARAM_SEARCHTIME"
;        "UDM_PARAM_SEARCH_TIME" "UDM_PARAM_FIRST_DOC"
;        "UDM_PARAM_LAST_DOC" "UDM_MODE_ALL" "UDM_MODE_ANY"
;        "UDM_MODE_BOOL" "UDM_MODE_PHRASE" "UDM_CACHE_ENABLED"
;        "UDM_CACHE_DISABLED" "UDM_TRACK_ENABLED" "UDM_TRACK_DISABLED"
;        "UDM_PHRASE_ENABLED" "UDM_PHRASE_DISABLED"
;        "UDM_CROSS_WORDS_ENABLED" "UDM_CROSSWORDS_ENABLED"
;        "UDM_CROSS_WORDS_DISABLED" "UDM_CROSSWORDS_DISABLED"
;        "UDM_PREFIXES_ENABLED" "UDM_PREFIX_ENABLED"
;        "UDM_ISPELL_PREFIXES_ENABLED" "UDM_ISPELL_PREFIX_ENABLED"
;        "UDM_PREFIXES_DISABLED" "UDM_PREFIX_DISABLED"
;        "UDM_ISPELL_PREFIXES_DISABLED" "UDM_ISPELL_PREFIX_DISABLED"
;        "UDM_ISPELL_TYPE_AFFIX" "UDM_ISPELL_TYPE_SPELL"
;        "UDM_ISPELL_TYPE_DB" "UDM_ISPELL_TYPE_SERVER" "UDM_MATCH_WORD"
;        "UDM_MATCH_BEGIN" "UDM_MATCH_SUBSTR" "UDM_MATCH_END"
;        "MSQL_ASSOC" "MSQL_NUM" "MSQL_BOTH" "MYSQL_ASSOC" "MYSQL_NUM"
;        "MYSQL_BOTH" "MYSQL_USE_RESULT" "MYSQL_STORE_RESULT"
;        "OCI_DEFAULT" "OCI_DESCRIBE_ONLY" "OCI_COMMIT_ON_SUCCESS"
;        "OCI_EXACT_FETCH" "SQLT_BFILEE" "SQLT_CFILEE" "SQLT_CLOB"
;        "SQLT_BLOB" "SQLT_RDD" "OCI_B_SQLT_NTY" "OCI_SYSDATE"
;        "OCI_B_BFILE" "OCI_B_CFILEE" "OCI_B_CLOB" "OCI_B_BLOB"
;        "OCI_B_ROWID" "OCI_B_CURSOR" "OCI_B_BIN" "OCI_ASSOC" "OCI_NUM"
;        "OCI_BOTH" "OCI_RETURN_NULLS" "OCI_RETURN_LOBS"
;        "OCI_DTYPE_FILE" "OCI_DTYPE_LOB" "OCI_DTYPE_ROWID" "OCI_D_FILE"
;        "OCI_D_LOB" "OCI_D_ROWID" "ODBC_TYPE" "ODBC_BINMODE_PASSTHRU"
;        "ODBC_BINMODE_RETURN" "ODBC_BINMODE_CONVERT" "SQL_ODBC_CURSORS"
;        "SQL_CUR_USE_DRIVER" "SQL_CUR_USE_IF_NEEDED" "SQL_CUR_USE_ODBC"
;        "SQL_CONCURRENCY" "SQL_CONCUR_READ_ONLY" "SQL_CONCUR_LOCK"
;        "SQL_CONCUR_ROWVER" "SQL_CONCUR_VALUES" "SQL_CURSOR_TYPE"
;        "SQL_CURSOR_FORWARD_ONLY" "SQL_CURSOR_KEYSET_DRIVEN"
;        "SQL_CURSOR_DYNAMIC" "SQL_CURSOR_STATIC" "SQL_KEYSET_SIZE"
;        "SQL_CHAR" "SQL_VARCHAR" "SQL_LONGVARCHAR" "SQL_DECIMAL"
;        "SQL_NUMERIC" "SQL_BIT" "SQL_TINYINT" "SQL_SMALLINT"
;        "SQL_INTEGER" "SQL_BIGINT" "SQL_REAL" "SQL_FLOAT" "SQL_DOUBLE"
;        "SQL_BINARY" "SQL_VARBINARY" "SQL_LONGVARBINARY" "SQL_DATE"
;        "SQL_TIME" "SQL_TIMESTAMP" "SQL_TYPE_DATE" "SQL_TYPE_TIME"
;        "SQL_TYPE_TIMESTAMP" "SQL_BEST_ROWID" "SQL_ROWVER"
;        "SQL_SCOPE_CURROW" "SQL_SCOPE_TRANSACTION" "SQL_SCOPE_SESSION"
;        "SQL_NO_NULLS" "SQL_NULLABLE" "SQL_INDEX_UNIQUE"
;        "SQL_INDEX_ALL" "SQL_ENSURE" "SQL_QUICK"
;        "X509_PURPOSE_SSL_CLIENT" "X509_PURPOSE_SSL_SERVER"
;        "X509_PURPOSE_NS_SSL_SERVER" "X509_PURPOSE_SMIME_SIGN"
;        "X509_PURPOSE_SMIME_ENCRYPT" "X509_PURPOSE_CRL_SIGN"
;        "X509_PURPOSE_ANY" "PKCS7_DETACHED" "PKCS7_TEXT"
;        "PKCS7_NOINTERN" "PKCS7_NOVERIFY" "PKCS7_NOCHAIN"
;        "PKCS7_NOCERTS" "PKCS7_NOATTR" "PKCS7_BINARY" "PKCS7_NOSIGS"
;        "OPENSSL_PKCS1_PADDING" "OPENSSL_SSLV23_PADDING"
;        "OPENSSL_NO_PADDING" "OPENSSL_PKCS1_OAEP_PADDING"
;        "ORA_BIND_INOUT" "ORA_BIND_IN" "ORA_BIND_OUT"
;        "ORA_FETCHINTO_ASSOC" "ORA_FETCHINTO_NULLS"
;        "PREG_PATTERN_ORDER" "PREG_SET_ORDER" "PREG_SPLIT_NO_EMPTY"
;        "PREG_SPLIT_DELIM_CAPTURE" "PGSQL_ASSOC" "PGSQL_NUM"
;        "PGSQL_BOTH" "PRINTER_COPIES" "PRINTER_MODE" "PRINTER_TITLE"
;        "PRINTER_DEVICENAME" "PRINTER_DRIVERVERSION"
;        "PRINTER_RESOLUTION_Y" "PRINTER_RESOLUTION_X" "PRINTER_SCALE"
;        "PRINTER_BACKGROUND_COLOR" "PRINTER_PAPER_LENGTH"
;        "PRINTER_PAPER_WIDTH" "PRINTER_PAPER_FORMAT"
;        "PRINTER_FORMAT_CUSTOM" "PRINTER_FORMAT_LETTER"
;        "PRINTER_FORMAT_LEGAL" "PRINTER_FORMAT_A3" "PRINTER_FORMAT_A4"
;        "PRINTER_FORMAT_A5" "PRINTER_FORMAT_B4" "PRINTER_FORMAT_B5"
;        "PRINTER_FORMAT_FOLIO" "PRINTER_ORIENTATION"
;        "PRINTER_ORIENTATION_PORTRAIT" "PRINTER_ORIENTATION_LANDSCAPE"
;        "PRINTER_TEXT_COLOR" "PRINTER_TEXT_ALIGN" "PRINTER_TA_BASELINE"
;        "PRINTER_TA_BOTTOM" "PRINTER_TA_TOP" "PRINTER_TA_CENTER"
;        "PRINTER_TA_LEFT" "PRINTER_TA_RIGHT" "PRINTER_PEN_SOLID"
;        "PRINTER_PEN_DASH" "PRINTER_PEN_DOT" "PRINTER_PEN_DASHDOT"
;        "PRINTER_PEN_DASHDOTDOT" "PRINTER_PEN_INVISIBLE"
;        "PRINTER_BRUSH_SOLID" "PRINTER_BRUSH_CUSTOM"
;        "PRINTER_BRUSH_DIAGONAL" "PRINTER_BRUSH_CROSS"
;        "PRINTER_BRUSH_DIAGCROSS" "PRINTER_BRUSH_FDIAGONAL"
;        "PRINTER_BRUSH_HORIZONTAL" "PRINTER_BRUSH_VERTICAL"
;        "PRINTER_FW_THIN" "PRINTER_FW_ULTRALIGHT" "PRINTER_FW_LIGHT"
;        "PRINTER_FW_NORMAL" "PRINTER_FW_MEDIUM" "PRINTER_FW_BOLD"
;        "PRINTER_FW_ULTRABOLD" "PRINTER_FW_HEAVY" "PRINTER_ENUM_LOCAL"
;        "PRINTER_ENUM_NAME" "PRINTER_ENUM_SHARED"
;        "PRINTER_ENUM_DEFAULT" "PRINTER_ENUM_CONNECTIONS"
;        "PRINTER_ENUM_NETWORK" "PRINTER_ENUM_REMOTE" "PSPELL_FAST"
;        "PSPELL_NORMAL" "PSPELL_BAD_SPELLERS" "PSPELL_RUN_TOGETHER"
;        "SID" "SID" "AF_UNIX" "AF_INET" "SOCK_STREAM" "SOCK_DGRAM"
;        "SOCK_RAW" "SOCK_SEQPACKET" "SOCK_RDM" "MSG_OOB" "MSG_WAITALL"
;        "MSG_PEEK" "MSG_DONTROUTE" "SO_DEBUG" "SO_REUSEADDR"
;        "SO_KEEPALIVE" "SO_DONTROUTE" "SO_LINGER" "SO_BROADCAST"
;        "SO_OOBINLINE" "SO_SNDBUF" "SO_RCVBUF" "SO_SNDLOWAT"
;        "SO_RCVLOWAT" "SO_SNDTIMEO" "SO_RCVTIMEO" "SO_TYPE" "SO_ERROR"
;        "SOL_SOCKET" "PHP_NORMAL_READ" "PHP_BINARY_READ"
;        "PHP_SYSTEM_READ" "SOL_TCP" "SOL_UDP" "MOD_COLOR" "MOD_MATRIX"
;        "TYPE_PUSHBUTTON" "TYPE_MENUBUTTON" "BSHitTest" "BSDown"
;        "BSOver" "BSUp" "OverDowntoIdle" "IdletoOverDown"
;        "OutDowntoIdle" "OutDowntoOverDown" "OverDowntoOutDown"
;        "OverUptoOverDown" "OverUptoIdle" "IdletoOverUp" "ButtonEnter"
;        "ButtonExit" "MenuEnter" "MenuExit" "XML_ERROR_NONE"
;        "XML_ERROR_NO_MEMORY" "XML_ERROR_SYNTAX"
;        "XML_ERROR_NO_ELEMENTS" "XML_ERROR_INVALID_TOKEN"
;        "XML_ERROR_UNCLOSED_TOKEN" "XML_ERROR_PARTIAL_CHAR"
;        "XML_ERROR_TAG_MISMATCH" "XML_ERROR_DUPLICATE_ATTRIBUTE"
;        "XML_ERROR_JUNK_AFTER_DOC_ELEMENT" "XML_ERROR_PARAM_ENTITY_REF"
;        "XML_ERROR_UNDEFINED_ENTITY" "XML_ERROR_RECURSIVE_ENTITY_REF"
;        "XML_ERROR_ASYNC_ENTITY" "XML_ERROR_BAD_CHAR_REF"
;        "XML_ERROR_BINARY_ENTITY_REF"
;        "XML_ERROR_ATTRIBUTE_EXTERNAL_ENTITY_REF"
;        "XML_ERROR_MISPLACED_XML_PI" "XML_ERROR_UNKNOWN_ENCODING"
;        "XML_ERROR_INCORRECT_ENCODING"
;        "XML_ERROR_UNCLOSED_CDATA_SECTION"
;        "XML_ERROR_EXTERNAL_ENTITY_HANDLING" "XML_OPTION_CASE_FOLDING"
;        "XML_OPTION_TARGET_ENCODING" "XML_OPTION_SKIP_TAGSTART"
;        "XML_OPTION_SKIP_WHITE" "YPERR_BADARGS" "YPERR_BADDB"
;        "YPERR_BUSY" "YPERR_DOMAIN" "YPERR_KEY" "YPERR_MAP"
;        "YPERR_NODOM" "YPERR_NOMORE" "YPERR_PMAP" "YPERR_RESRC"
;        "YPERR_RPC" "YPERR_YPBIND" "YPERR_YPERR" "YPERR_YPSERV"
;        "YPERR_VERS" "FORCE_GZIP" "FORCE_DEFLATE"

       ;; PEAR constants
;        "PEAR_ERROR_RETURN" "PEAR_ERROR_PRINT" "PEAR_ERROR_TRIGGER"
;        "PEAR_ERROR_DIE" "PEAR_ERROR_CALLBACK" "OS_WINDOWS" "OS_UNIX"
;        "PEAR_OS" "DB_OK" "DB_ERROR" "DB_ERROR_SYNTAX"
;        "DB_ERROR_CONSTRAINT" "DB_ERROR_NOT_FOUND"
;        "DB_ERROR_ALREADY_EXISTS" "DB_ERROR_UNSUPPORTED"
;        "DB_ERROR_MISMATCH" "DB_ERROR_INVALID" "DB_ERROR_NOT_CAPABLE"
;        "DB_ERROR_TRUNCATED" "DB_ERROR_INVALID_NUMBER"
;        "DB_ERROR_INVALID_DATE" "DB_ERROR_DIVZERO"
;        "DB_ERROR_NODBSELECTED" "DB_ERROR_CANNOT_CREATE"
;        "DB_ERROR_CANNOT_DELETE" "DB_ERROR_CANNOT_DROP"
;        "DB_ERROR_NOSUCHTABLE" "DB_ERROR_NOSUCHFIELD"
;        "DB_ERROR_NEED_MORE_DATA" "DB_ERROR_NOT_LOCKED"
;        "DB_ERROR_VALUE_COUNT_ON_ROW" "DB_ERROR_INVALID_DSN"
;        "DB_ERROR_CONNECT_FAILED" "DB_WARNING" "DB_WARNING_READ_ONLY"
;        "DB_PARAM_SCALAR" "DB_PARAM_OPAQUE" "DB_BINMODE_PASSTHRU"
;        "DB_BINMODE_RETURN" "DB_BINMODE_CONVERT" "DB_FETCHMODE_DEFAULT"
;        "DB_FETCHMODE_ORDERED" "DB_FETCHMODE_ASSOC"
;        "DB_FETCHMODE_FLIPPED" "DB_GETMODE_ORDERED" "DB_GETMODE_ASSOC"
;        "DB_GETMODE_FLIPPED" "DB_TABLEINFO_ORDER"
;        "DB_TABLEINFO_ORDERTABLE" "DB_TABLEINFO_FULL"
       )
  "PHP constants.")

(defconst php-keywords
  ;; "class", "new" and "extends" get special treatment
  ;; "case" and "default" get special treatment elsewhere
  '("xor" "while" "var" "try" "throw" "then" "switch" "static" "return"
    "require_once" "require" "private" "or" "next" "instanceof"
    "include_once" "include" "if" "global" "foreach" "for" "finally"
    "extends" "exit" "endwhile" "endswitch" "endif" "endforeach" "endfor"
    "elseif" "else if" "else" "echo" "do" "declare" "continue" "catch all"
    "catch" "break" "as" "and")
  "PHP keywords.")

(defconst php-functions
  ;; internal functions
  '("zlib_get_coding_type" "zip_read" "zip_open" "zip_entry_read"
    "zip_entry_open" "zip_entry_name" "zip_entry_filesize"
    "zip_entry_compressionmethod" "zip_entry_compressedsize"
    "zip_entry_close" "zip_close" "zend_version" "zend_logo_guid"
    "yp_order" "yp_next" "yp_match" "yp_master" "yp_get_default_domain"
    "yp_first" "yp_errno" "yp_err_string" "yp_cat" "yp_all" "yaz_wait"
    "yaz_syntax" "yaz_sort" "yaz_set_option" "yaz_search" "yaz_schema"
    "yaz_scan_result" "yaz_scan" "yaz_record" "yaz_range" "yaz_present"
    "yaz_itemorder" "yaz_hits" "yaz_get_option" "yaz_es_result"
    "yaz_error" "yaz_errno" "yaz_element" "yaz_database" "yaz_connect"
    "yaz_close" "yaz_ccl_parse" "yaz_ccl_conf" "yaz_addinfo" "xslt_setopt"
    "xslt_set_scheme_handlers" "xslt_set_scheme_handler"
    "xslt_set_sax_handlers" "xslt_set_sax_handler" "xslt_set_object"
    "xslt_set_log" "xslt_set_error_handler" "xslt_set_encoding"
    "xslt_set_base" "xslt_process" "xslt_getopt" "xslt_free" "xslt_error"
    "xslt_errno" "xslt_create" "xslt_backend_version" "xslt_backend_name"
    "xslt_backend_info" "xsl_xsltprocessor_transform_to_xml"
    "xsl_xsltprocessor_transform_to_uri"
    "xsl_xsltprocessor_transform_to_doc" "xsl_xsltprocessor_set_parameter"
    "xsl_xsltprocessor_remove_parameter"
    "xsl_xsltprocessor_register_php_functions"
    "xsl_xsltprocessor_import_stylesheet"
    "xsl_xsltprocessor_has_exslt_support"
    "xsl_xsltprocessor_get_parameter" "xptr_new_context" "xptr_eval"
    "xpath_new_context" "xpath_eval_expression" "xpath_eval"
    "xmlrpc_set_type" "xmlrpc_server_register_method"
    "xmlrpc_server_register_introspection_callback"
    "xmlrpc_server_destroy" "xmlrpc_server_create"
    "xmlrpc_server_call_method" "xmlrpc_server_add_introspection_data"
    "xmlrpc_parse_method_descriptions" "xmlrpc_is_fault" "xmlrpc_get_type"
    "xmlrpc_encode_request" "xmlrpc_encode" "xmlrpc_decode_request"
    "xmlrpc_decode" "xml_set_unparsed_entity_decl_handler"
    "xml_set_start_namespace_decl_handler"
    "xml_set_processing_instruction_handler" "xml_set_object"
    "xml_set_notation_decl_handler" "xml_set_external_entity_ref_handler"
    "xml_set_end_namespace_decl_handler" "xml_set_element_handler"
    "xml_set_default_handler" "xml_set_character_data_handler"
    "xml_parser_set_option" "xml_parser_get_option" "xml_parser_free"
    "xml_parser_create_ns" "xml_parser_create" "xml_parse_into_struct"
    "xml_parse" "xml_get_error_code" "xml_get_current_line_number"
    "xml_get_current_column_number" "xml_get_current_byte_index"
    "xml_error_string" "xdiff_string_patch_binary" "xdiff_string_patch"
    "xdiff_string_merge3" "xdiff_string_diff_binary" "xdiff_string_diff"
    "xdiff_file_patch_binary" "xdiff_file_patch" "xdiff_file_merge3"
    "xdiff_file_diff_binary" "xdiff_file_diff" "xattr_supported"
    "xattr_set" "xattr_remove" "xattr_list" "xattr_get" "wordwrap"
    "wddx_serialize_vars" "wddx_serialize_value" "wddx_packet_start"
    "wddx_packet_end" "wddx_deserialize" "wddx_add_vars"
    "w32api_set_call_method" "w32api_register_function"
    "w32api_invoke_function" "w32api_init_dtype" "w32api_deftype"
    "vsprintf" "vprintf" "vpopmail_set_user_quota" "vpopmail_passwd"
    "vpopmail_error" "vpopmail_del_user" "vpopmail_del_domain_ex"
    "vpopmail_del_domain" "vpopmail_auth_user" "vpopmail_alias_get_all"
    "vpopmail_alias_get" "vpopmail_alias_del_domain" "vpopmail_alias_del"
    "vpopmail_alias_add" "vpopmail_add_user" "vpopmail_add_domain_ex"
    "vpopmail_add_domain" "vpopmail_add_alias_domain_ex"
    "vpopmail_add_alias_domain" "virtual" "vfprintf" "version_compare"
    "variant_xor" "variant_sub" "variant_set_type" "variant_set"
    "variant_round" "variant_pow" "variant_or" "variant_not" "variant_neg"
    "variant_mul" "variant_mod" "variant_int" "variant_imp" "variant_idiv"
    "variant_get_type" "variant_fix" "variant_eqv" "variant_div"
    "variant_date_to_timestamp" "variant_date_from_timestamp"
    "variant_cmp" "variant_cat" "variant_cast" "variant_and" "variant_add"
    "variant_abs" "var_export" "var_dump" "utf8_encode" "utf8_decode"
    "usort" "usleep" "user_error" "urlencode" "urldecode" "unset"
    "unserialize" "unregister_tick_function" "unpack" "unlink" "unixtojd"
    "uniqid" "umask" "uksort" "udm_set_agent_param" "udm_open_stored"
    "udm_load_ispell_data" "udm_hash32" "udm_get_res_param"
    "udm_get_res_field" "udm_get_doc_count" "udm_free_res"
    "udm_free_ispell_data" "udm_free_agent" "udm_find" "udm_error"
    "udm_errno" "udm_crc32" "udm_close_stored" "udm_clear_search_limits"
    "udm_check_stored" "udm_check_charset" "udm_cat_path" "udm_cat_list"
    "udm_api_version" "udm_alloc_agent_array" "udm_alloc_agent"
    "udm_add_search_limit" "ucwords" "ucfirst" "uasort" "trim"
    "trigger_error" "touch" "token_name" "token_get_all" "tmpfile"
    "time_nanosleep" "time" "tidy_warning_count" "tidy_setopt"
    "tidy_set_encoding" "tidy_save_config" "tidy_reset_config"
    "tidy_repair_string" "tidy_repair_file" "tidy_parse_string"
    "tidy_parse_file" "tidy_node_tidy_node" "tidy_node_prev"
    "tidy_node_next" "tidy_node_isxml" "tidy_node_isxhtml"
    "tidy_node_istext" "tidy_node_isphp" "tidy_node_isjste"
    "tidy_node_ishtml" "tidy_node_iscomment" "tidy_node_isasp"
    "tidy_node_hassiblings" "tidy_node_haschildren" "tidy_node_get_nodes"
    "tidy_node_get_attr" "tidy_node_children" "tidy_node_attributes"
    "tidy_load_config" "tidy_is_xml" "tidy_is_xhtml" "tidy_getopt"
    "tidy_get_status" "tidy_get_root" "tidy_get_release" "tidy_get_output"
    "tidy_get_html_ver" "tidy_get_html" "tidy_get_head"
    "tidy_get_error_buffer" "tidy_get_config" "tidy_get_body"
    "tidy_error_count" "tidy_diagnose" "tidy_construct"
    "tidy_config_count" "tidy_clean_repair" "tidy_access_count"
    "textdomain" "tempnam" "tcpwrap_check" "tanh" "tan" "system" "syslog"
    "symlink" "sybase_unbuffered_query" "sybase_set_message_handler"
    "sybase_select_db" "sybase_result" "sybase_query" "sybase_pconnect"
    "sybase_num_rows" "sybase_num_fields" "sybase_min_server_severity"
    "sybase_min_message_severity" "sybase_min_error_severity"
    "sybase_min_client_severity" "sybase_get_last_message"
    "sybase_free_result" "sybase_field_seek" "sybase_fetch_row"
    "sybase_fetch_object" "sybase_fetch_field" "sybase_fetch_assoc"
    "sybase_fetch_array" "sybase_deadlock_retry_count" "sybase_data_seek"
    "sybase_connect" "sybase_close" "sybase_affected_rows"
    "swftextfield.setrightmargin" "swftextfield.setname"
    "swftextfield.setmargins" "swftextfield.setlinespacing"
    "swftextfield.setleftmargin" "swftextfield.setindentation"
    "swftextfield.setheight" "swftextfield.setfont"
    "swftextfield.setcolor" "swftextfield.setbounds" "swftextfield.align"
    "swftextfield.addstring" "swftextfield" "swftext.setspacing"
    "swftext.setheight" "swftext.setfont" "swftext.setcolor"
    "swftext.moveto" "swftext.getwidth" "swftext.addstring" "swftext"
    "swfsprite.setframes" "swfsprite.remove" "swfsprite.nextframe"
    "swfsprite.add" "swfsprite" "swfshape.setrightfill" "swfshape.setline"
    "swfshape.setleftfill" "swfshape.movepento" "swfshape.movepen"
    "swfshape.drawlineto" "swfshape.drawline" "swfshape.drawcurveto"
    "swfshape.drawcurve" "swfshape.addfill" "swfshape"
    "swfmovie.streammp3" "swfmovie.setrate" "swfmovie.setframes"
    "swfmovie.setdimension" "swfmovie.setbackground" "swfmovie.save"
    "swfmovie.remove" "swfmovie.output" "swfmovie.nextframe"
    "swfmovie.add" "swfmovie" "swfmorph.getshape2" "swfmorph.getshape1"
    "swfmorph" "swfgradient.addentry" "swfgradient" "swffont.getwidth"
    "swffont" "swffill.skewyto" "swffill.skewxto" "swffill.scaleto"
    "swffill.rotateto" "swffill.moveto" "swffill" "swfdisplayitem.skewyto"
    "swfdisplayitem.skewy" "swfdisplayitem.skewxto" "swfdisplayitem.skewx"
    "swfdisplayitem.setratio" "swfdisplayitem.setname"
    "swfdisplayitem.setdepth" "swfdisplayitem.scaleto"
    "swfdisplayitem.scale" "swfdisplayitem.rotateto"
    "swfdisplayitem.rotate" "swfdisplayitem.remove"
    "swfdisplayitem.multcolor" "swfdisplayitem.moveto"
    "swfdisplayitem.move" "swfdisplayitem.addcolor" "swfdisplayitem"
    "swfbutton.setup" "swfbutton.setover" "swfbutton.sethit"
    "swfbutton.setdown" "swfbutton.setaction" "swfbutton.addshape"
    "swfbutton.addaction" "swfbutton_keypress" "swfbutton"
    "swfbitmap.getwidth" "swfbitmap.getheight" "swfbitmap" "swfaction"
    "swf_viewport" "swf_translate" "swf_textwidth" "swf_startsymbol"
    "swf_startshape" "swf_startdoaction" "swf_startbutton" "swf_showframe"
    "swf_shapemoveto" "swf_shapelineto" "swf_shapelinesolid"
    "swf_shapefillsolid" "swf_shapefilloff" "swf_shapefillbitmaptile"
    "swf_shapefillbitmapclip" "swf_shapecurveto3" "swf_shapecurveto"
    "swf_shapearc" "swf_setframe" "swf_setfont" "swf_scale" "swf_rotate"
    "swf_removeobject" "swf_pushmatrix" "swf_posround" "swf_popmatrix"
    "swf_polarview" "swf_placeobject" "swf_perspective" "swf_ortho2"
    "swf_ortho" "swf_openfile" "swf_oncondition" "swf_nextid"
    "swf_mulcolor" "swf_modifyobject" "swf_lookat" "swf_labelframe"
    "swf_getframe" "swf_getfontinfo" "swf_getbitmapinfo"
    "swf_fonttracking" "swf_fontslant" "swf_fontsize" "swf_endsymbol"
    "swf_endshape" "swf_enddoaction" "swf_endbutton" "swf_definetext"
    "swf_definerect" "swf_definepoly" "swf_defineline" "swf_definefont"
    "swf_definebitmap" "swf_closefile" "swf_addcolor"
    "swf_addbuttonrecord" "swf_actionwaitforframe"
    "swf_actiontogglequality" "swf_actionstop" "swf_actionsettarget"
    "swf_actionprevframe" "swf_actionplay" "swf_actionnextframe"
    "swf_actiongotolabel" "swf_actiongotoframe" "swf_actiongeturl"
    "substr_replace" "substr_count" "substr_compare" "substr" "strval"
    "strtr" "strtoupper" "strtotime" "strtolower" "strtok" "strstr"
    "strspn" "strrpos" "strripos" "strrev" "strrchr" "strpos" "strpbrk"
    "strncmp" "strncasecmp" "strnatcmp" "strnatcasecmp" "strlen" "stristr"
    "stripslashes" "stripos" "stripcslashes" "strip_tags" "strftime"
    "stream_wrapper_register" "stream_socket_server"
    "stream_socket_sendto" "stream_socket_recvfrom"
    "stream_socket_get_name" "stream_socket_enable_crypto"
    "stream_socket_client" "stream_socket_accept"
    "stream_set_write_buffer" "stream_set_timeout" "stream_set_blocking"
    "stream_select" "stream_register_wrapper" "stream_get_wrappers"
    "stream_get_transports" "stream_get_meta_data" "stream_get_line"
    "stream_get_filters" "stream_get_contents" "stream_filter_register"
    "stream_filter_prepend" "stream_filter_append" "stream_copy_to_stream"
    "stream_context_set_params" "stream_context_set_option"
    "stream_context_get_options" "stream_context_get_default"
    "stream_context_create" "strcspn" "strcoll" "strcmp" "strchr"
    "strcasecmp" "str_word_count" "str_split" "str_shuffle" "str_rot13"
    "str_replace" "str_repeat" "str_pad" "str_ireplace" "stat" "sscanf"
    "srand" "sqrt" "sqlite_unbuffered_query" "sqlite_udf_encode_binary"
    "sqlite_udf_decode_binary" "sqlite_single_query" "sqlite_seek"
    "sqlite_rewind" "sqlite_query" "sqlite_prev" "sqlite_popen"
    "sqlite_open" "sqlite_num_rows" "sqlite_num_fields" "sqlite_next"
    "sqlite_libversion" "sqlite_libencoding" "sqlite_last_insert_rowid"
    "sqlite_last_error" "sqlite_has_prev" "sqlite_has_more"
    "sqlite_field_name" "sqlite_fetch_string" "sqlite_fetch_single"
    "sqlite_fetch_object" "sqlite_fetch_column_types" "sqlite_fetch_array"
    "sqlite_fetch_all" "sqlite_factory" "sqlite_exec"
    "sqlite_escape_string" "sqlite_error_string" "sqlite_current"
    "sqlite_create_function" "sqlite_create_aggregate" "sqlite_column"
    "sqlite_close" "sqlite_changes" "sqlite_busy_timeout"
    "sqlite_array_query" "sql_regcase" "sprintf" "spliti" "split"
    "soundex" "sort" "socket_write" "socket_strerror" "socket_shutdown"
    "socket_set_timeout" "socket_set_option" "socket_set_nonblock"
    "socket_set_blocking" "socket_set_block" "socket_sendto" "socket_send"
    "socket_select" "socket_recvfrom" "socket_recv" "socket_read"
    "socket_listen" "socket_last_error" "socket_getsockname"
    "socket_getpeername" "socket_get_status" "socket_get_option"
    "socket_create_pair" "socket_create_listen" "socket_create"
    "socket_connect" "socket_close" "socket_clear_error" "socket_bind"
    "socket_accept" "soapvar_soapvar" "soapserver_soapserver"
    "soapserver_setpersistence" "soapserver_setclass" "soapserver_handle"
    "soapserver_getfunctions" "soapserver_addfunction"
    "soapparam_soapparam" "soapheader_soapheader" "soapfault_soapfault"
    "soapclient_soapclient" "soapclient_gettypes"
    "soapclient_getlastresponse" "soapclient_getlastrequest"
    "soapclient_getfunctions" "soapclient_call" "snmpwalkoid" "snmpwalk"
    "snmpset" "snmprealwalk" "snmpgetnext" "snmpget"
    "snmp_set_valueretrieval" "snmp_set_quick_print"
    "snmp_set_oid_numeric_print" "snmp_set_enum_print" "snmp_read_mib"
    "snmp_get_valueretrieval" "snmp_get_quick_print" "sleep" "sizeof"
    "sinh" "sin" "simplexmliterator_valid" "simplexmliterator_rewind"
    "simplexmliterator_next" "simplexmliterator_key"
    "simplexmliterator_haschildren" "simplexmliterator_getchildren"
    "simplexmliterator_current" "simplexml_load_string"
    "simplexml_load_file" "simplexml_import_dom" "simplexml_element_xpath"
    "simplexml_element_children" "simplexml_element_attributes"
    "simplexml_element_asxml" "similar_text" "shuffle" "show_source"
    "shmop_write" "shmop_size" "shmop_read" "shmop_open" "shmop_delete"
    "shmop_close" "shm_remove_var" "shm_remove" "shm_put_var"
    "shm_get_var" "shm_detach" "shm_attach" "shell_exec" "sha1_file"
    "sha1" "settype" "setrawcookie" "setlocale" "setcookie"
    "set_time_limit" "set_magic_quotes_runtime" "set_include_path"
    "set_file_buffer" "set_exception_handler" "set_error_handler"
    "session_write_close" "session_unset" "session_unregister"
    "session_start" "session_set_save_handler" "session_set_cookie_params"
    "session_save_path" "session_register" "session_regenerate_id"
    "session_name" "session_module_name" "session_is_registered"
    "session_id" "session_get_cookie_params" "session_encode"
    "session_destroy" "session_decode" "session_commit"
    "session_cache_limiter" "session_cache_expire" "sesam_settransaction"
    "sesam_seek_row" "sesam_rollback" "sesam_query" "sesam_num_fields"
    "sesam_free_result" "sesam_field_name" "sesam_field_array"
    "sesam_fetch_row" "sesam_fetch_result" "sesam_fetch_array"
    "sesam_execimm" "sesam_errormsg" "sesam_disconnect" "sesam_diagnostic"
    "sesam_connect" "sesam_commit" "sesam_affected_rows" "serialize"
    "sem_remove" "sem_release" "sem_get" "sem_acquire" "scandir" "rtrim"
    "rsort" "round" "rmdir" "rewinddir" "rewind" "return"
    "restore_include_path" "restore_exception_handler"
    "restore_error_handler" "reset" "require_once" "require" "rename"
    "register_tick_function" "register_shutdown_function"
    "recursiveiteratoriterator_valid" "recursiveiteratoriterator_rewind"
    "recursiveiteratoriterator_next" "recursiveiteratoriterator_key"
    "recursiveiteratoriterator_getsubiterator"
    "recursiveiteratoriterator_getdepth"
    "recursiveiteratoriterator_current"
    "recursivedirectoryiterator_rewind" "recursivedirectoryiterator_next"
    "recursivedirectoryiterator_key"
    "recursivedirectoryiterator_haschildren"
    "recursivedirectoryiterator_getchildren" "recode_string" "recode_file"
    "recode" "realpath" "readlink" "readline_write_history"
    "readline_read_history" "readline_list_history" "readline_info"
    "readline_completion_function" "readline_clear_history"
    "readline_callback_read_char" "readline_callback_handler_remove"
    "readline_callback_handler_install" "readline_add_history" "readline"
    "readgzfile" "readfile" "readdir" "read_exif_data" "rawurlencode"
    "rawurldecode" "range" "rand" "rad2deg" "quotemeta"
    "quoted_printable_decode" "qdom_tree" "qdom_error" "putenv"
    "pspell_suggest" "pspell_store_replacement" "pspell_save_wordlist"
    "pspell_new_personal" "pspell_new_config" "pspell_new"
    "pspell_config_save_repl" "pspell_config_runtogether"
    "pspell_config_repl" "pspell_config_personal" "pspell_config_mode"
    "pspell_config_ignore" "pspell_config_dict_dir"
    "pspell_config_data_dir" "pspell_config_create" "pspell_clear_session"
    "pspell_check" "pspell_add_to_session" "pspell_add_to_personal"
    "proc_terminate" "proc_open" "proc_nice" "proc_get_status"
    "proc_close" "printf" "printer_write" "printer_start_page"
    "printer_start_doc" "printer_set_option" "printer_select_pen"
    "printer_select_font" "printer_select_brush" "printer_open"
    "printer_logical_fontheight" "printer_list" "printer_get_option"
    "printer_end_page" "printer_end_doc" "printer_draw_text"
    "printer_draw_roundrect" "printer_draw_rectangle" "printer_draw_pie"
    "printer_draw_line" "printer_draw_elipse" "printer_draw_chord"
    "printer_draw_bmp" "printer_delete_pen" "printer_delete_font"
    "printer_delete_dc" "printer_delete_brush" "printer_create_pen"
    "printer_create_font" "printer_create_dc" "printer_create_brush"
    "printer_close" "printer_abort" "print_r" "print" "prev" "preg_split"
    "preg_replace_callback" "preg_replace" "preg_quote" "preg_match_all"
    "preg_match" "preg_grep" "pow" "posix_uname" "posix_ttyname"
    "posix_times" "posix_strerror" "posix_setuid" "posix_setsid"
    "posix_setpgid" "posix_setgid" "posix_seteuid" "posix_setegid"
    "posix_mkfifo" "posix_kill" "posix_isatty" "posix_getuid"
    "posix_getsid" "posix_getrlimit" "posix_getpwuid" "posix_getpwnam"
    "posix_getppid" "posix_getpid" "posix_getpgrp" "posix_getpgid"
    "posix_getlogin" "posix_getgroups" "posix_getgrnam" "posix_getgrgid"
    "posix_getgid" "posix_geteuid" "posix_getegid" "posix_getcwd"
    "posix_get_last_error" "posix_ctermid" "pos" "popen" "png2wbmp" "pi"
    "phpversion" "phpinfo" "phpcredits" "php_uname" "php_strip_whitespace"
    "php_sapi_name" "php_logo_guid" "php_ini_scanned_files"
    "php_check_syntax" "pg_version" "pg_update" "pg_untrace"
    "pg_unescape_bytea" "pg_tty" "pg_trace" "pg_set_client_encoding"
    "pg_send_query" "pg_select" "pg_result_status" "pg_result_seek"
    "pg_result_error" "pg_query" "pg_put_line" "pg_port" "pg_ping"
    "pg_pconnect" "pg_parameter_status" "pg_options" "pg_num_rows"
    "pg_num_fields" "pg_meta_data" "pg_lo_write" "pg_lo_unlink"
    "pg_lo_tell" "pg_lo_seek" "pg_lo_read_all" "pg_lo_read" "pg_lo_open"
    "pg_lo_import" "pg_lo_export" "pg_lo_create" "pg_lo_close"
    "pg_last_oid" "pg_last_notice" "pg_last_error" "pg_insert" "pg_host"
    "pg_get_result" "pg_get_pid" "pg_get_notify" "pg_free_result"
    "pg_field_type" "pg_field_size" "pg_field_prtlen" "pg_field_num"
    "pg_field_name" "pg_field_is_null" "pg_fetch_row" "pg_fetch_result"
    "pg_fetch_object" "pg_fetch_assoc" "pg_fetch_array" "pg_fetch_all"
    "pg_escape_string" "pg_escape_bytea" "pg_end_copy" "pg_delete"
    "pg_dbname" "pg_copy_to" "pg_copy_from" "pg_convert"
    "pg_connection_status" "pg_connection_reset" "pg_connection_busy"
    "pg_connect" "pg_close" "pg_client_encoding" "pg_cancel_query"
    "pg_affected_rows" "pfsockopen" "pfpro_version" "pfpro_process_raw"
    "pfpro_process" "pfpro_init" "pfpro_cleanup" "pdf_translate"
    "pdf_stroke" "pdf_stringwidth" "pdf_skew" "pdf_show_xy"
    "pdf_show_boxed" "pdf_show" "pdf_setrgbcolor_stroke"
    "pdf_setrgbcolor_fill" "pdf_setrgbcolor" "pdf_setpolydash"
    "pdf_setmiterlimit" "pdf_setmatrix" "pdf_setlinewidth"
    "pdf_setlinejoin" "pdf_setlinecap" "pdf_setgray_stroke"
    "pdf_setgray_fill" "pdf_setgray" "pdf_setfont" "pdf_setflat"
    "pdf_setdash" "pdf_setcolor" "pdf_set_word_spacing" "pdf_set_value"
    "pdf_set_text_rise" "pdf_set_text_rendering" "pdf_set_text_pos"
    "pdf_set_text_matrix" "pdf_set_parameter" "pdf_set_leading"
    "pdf_set_info_title" "pdf_set_info_subject" "pdf_set_info_keywords"
    "pdf_set_info_creator" "pdf_set_info_author" "pdf_set_info"
    "pdf_set_horiz_scaling" "pdf_set_font" "pdf_set_duration"
    "pdf_set_char_spacing" "pdf_set_border_style" "pdf_set_border_dash"
    "pdf_set_border_color" "pdf_scale" "pdf_save" "pdf_rotate"
    "pdf_restore" "pdf_rect" "pdf_place_pdi_page" "pdf_place_image"
    "pdf_open_tiff" "pdf_open_png" "pdf_open_pdi_page" "pdf_open_pdi"
    "pdf_open_memory_image" "pdf_open_jpeg" "pdf_open_image_file"
    "pdf_open_image" "pdf_open_gif" "pdf_open_file" "pdf_open_ccitt"
    "pdf_open" "pdf_new" "pdf_moveto" "pdf_makespotcolor" "pdf_lineto"
    "pdf_initgraphics" "pdf_get_value" "pdf_get_pdi_value"
    "pdf_get_pdi_parameter" "pdf_get_parameter" "pdf_get_minorversion"
    "pdf_get_majorversion" "pdf_get_image_width" "pdf_get_image_height"
    "pdf_get_fontsize" "pdf_get_fontname" "pdf_get_font" "pdf_get_buffer"
    "pdf_findfont" "pdf_fill_stroke" "pdf_fill" "pdf_endpath"
    "pdf_end_template" "pdf_end_pattern" "pdf_end_page" "pdf_delete"
    "pdf_curveto" "pdf_continue_text" "pdf_concat" "pdf_closepath_stroke"
    "pdf_closepath_fill_stroke" "pdf_closepath" "pdf_close_pdi_page"
    "pdf_close_pdi" "pdf_close_image" "pdf_close" "pdf_clip" "pdf_circle"
    "pdf_begin_template" "pdf_begin_pattern" "pdf_begin_page"
    "pdf_attach_file" "pdf_arcn" "pdf_arc" "pdf_add_weblink"
    "pdf_add_thumbnail" "pdf_add_pdflink" "pdf_add_outline" "pdf_add_note"
    "pdf_add_locallink" "pdf_add_launchlink" "pdf_add_bookmark"
    "pdf_add_annotation" "pcntl_wtermsig" "pcntl_wstopsig"
    "pcntl_wifstopped" "pcntl_wifsignaled" "pcntl_wifexited"
    "pcntl_wexitstatus" "pcntl_waitpid" "pcntl_wait" "pcntl_signal"
    "pcntl_setpriority" "pcntl_getpriority" "pcntl_fork" "pcntl_exec"
    "pcntl_alarm" "pclose" "pathinfo" "passthru" "parsekit_func_arginfo"
    "parsekit_compile_string" "parsekit_compile_file" "parse_url"
    "parse_str" "parse_ini_file" "parentiterator_rewind"
    "parentiterator_next" "parentiterator_haschildren"
    "parentiterator_getchildren" "pack" "ovrimos_rollback"
    "ovrimos_result_all" "ovrimos_result" "ovrimos_prepare"
    "ovrimos_num_rows" "ovrimos_num_fields" "ovrimos_longreadlen"
    "ovrimos_free_result" "ovrimos_field_type" "ovrimos_field_num"
    "ovrimos_field_name" "ovrimos_field_len" "ovrimos_fetch_row"
    "ovrimos_fetch_into" "ovrimos_execute" "ovrimos_exec" "ovrimos_cursor"
    "ovrimos_connect" "ovrimos_commit" "ovrimos_close" "overload"
    "output_reset_rewrite_vars" "output_add_rewrite_var" "ord"
    "ora_rollback" "ora_plogon" "ora_parse" "ora_open" "ora_numrows"
    "ora_numcols" "ora_logon" "ora_logoff" "ora_getcolumn"
    "ora_fetch_into" "ora_fetch" "ora_exec" "ora_errorcode" "ora_error"
    "ora_do" "ora_commiton" "ora_commitoff" "ora_commit" "ora_columntype"
    "ora_columnsize" "ora_columnname" "ora_close" "ora_bind"
    "openssl_x509_read" "openssl_x509_parse" "openssl_x509_free"
    "openssl_x509_export_to_file" "openssl_x509_export"
    "openssl_x509_checkpurpose" "openssl_x509_check_private_key"
    "openssl_verify" "openssl_sign" "openssl_seal"
    "openssl_public_encrypt" "openssl_public_decrypt"
    "openssl_private_encrypt" "openssl_private_decrypt" "openssl_pkey_new"
    "openssl_pkey_get_public" "openssl_pkey_get_private"
    "openssl_pkey_export_to_file" "openssl_pkey_export"
    "openssl_pkcs7_verify" "openssl_pkcs7_sign" "openssl_pkcs7_encrypt"
    "openssl_pkcs7_decrypt" "openssl_open" "openssl_get_publickey"
    "openssl_get_privatekey" "openssl_free_key" "openssl_error_string"
    "openssl_csr_sign" "openssl_csr_new" "openssl_csr_export_to_file"
    "openssl_csr_export" "openlog" "opendir" "odbc_tables"
    "odbc_tableprivileges" "odbc_statistics" "odbc_specialcolumns"
    "odbc_setoption" "odbc_rollback" "odbc_result_all" "odbc_result"
    "odbc_procedures" "odbc_procedurecolumns" "odbc_primarykeys"
    "odbc_prepare" "odbc_pconnect" "odbc_num_rows" "odbc_num_fields"
    "odbc_next_result" "odbc_longreadlen" "odbc_gettypeinfo"
    "odbc_free_result" "odbc_foreignkeys" "odbc_field_type"
    "odbc_field_scale" "odbc_field_precision" "odbc_field_num"
    "odbc_field_name" "odbc_field_len" "odbc_fetch_row"
    "odbc_fetch_object" "odbc_fetch_into" "odbc_fetch_array"
    "odbc_execute" "odbc_exec" "odbc_errormsg" "odbc_error" "odbc_do"
    "odbc_data_source" "odbc_cursor" "odbc_connect" "odbc_commit"
    "odbc_columns" "odbc_columnprivileges" "odbc_close_all" "odbc_close"
    "odbc_binmode" "odbc_autocommit" "octdec" "ociwritetemporarylob"
    "ociwritelobtofile" "ocistatementtype" "ocisetprefetch"
    "ocisetbufferinglob" "ociserverversion" "ocisavelobfile" "ocisavelob"
    "ocirowcount" "ocirollback" "ociresult" "ociplogon" "ociparse"
    "ocinumcols" "ocinlogon" "ocinewdescriptor" "ocinewcursor"
    "ocinewcollection" "ocilogon" "ocilogoff" "ociloadlob"
    "ociinternaldebug" "ocigetbufferinglob" "ocifreestatement"
    "ocifreedesc" "ocifreecursor" "ocifreecollection" "ocifetchstatement"
    "ocifetchinto" "ocifetch" "ociexecute" "ocierror" "ocidefinebyname"
    "ocicommit" "ocicolumntyperaw" "ocicolumntype" "ocicolumnsize"
    "ocicolumnscale" "ocicolumnprecision" "ocicolumnname"
    "ocicolumnisnull" "ocicolltrim" "ocicollsize" "ocicollmax"
    "ocicollgetelem" "ocicollassignelem" "ocicollassign" "ocicollappend"
    "ocicloselob" "ocicancel" "ocibindbyname" "oci_statement_type"
    "oci_set_prefetch" "oci_server_version" "oci_rollback" "oci_result"
    "oci_pconnect" "oci_password_change" "oci_parse" "oci_num_rows"
    "oci_num_fields" "oci_new_descriptor" "oci_new_cursor"
    "oci_new_connect" "oci_new_collection" "oci_lob_write_temporary"
    "oci_lob_write" "oci_lob_truncate" "oci_lob_tell" "oci_lob_size"
    "oci_lob_seek" "oci_lob_save" "oci_lob_rewind" "oci_lob_read"
    "oci_lob_load" "oci_lob_is_equal" "oci_lob_import" "oci_lob_flush"
    "oci_lob_export" "oci_lob_erase" "oci_lob_eof" "oci_lob_copy"
    "oci_lob_close" "oci_lob_append" "oci_internal_debug"
    "oci_free_statement" "oci_free_descriptor" "oci_free_collection"
    "oci_field_type_raw" "oci_field_type" "oci_field_size"
    "oci_field_scale" "oci_field_precision" "oci_field_name"
    "oci_field_is_null" "oci_fetch_row" "oci_fetch_object"
    "oci_fetch_assoc" "oci_fetch_array" "oci_fetch_all" "oci_fetch"
    "oci_execute" "oci_error" "oci_define_by_name" "oci_connect"
    "oci_commit" "oci_collection_trim" "oci_collection_size"
    "oci_collection_max" "oci_collection_element_get"
    "oci_collection_element_assign" "oci_collection_assign"
    "oci_collection_append" "oci_close" "oci_cancel" "oci_bind_by_name"
    "ob_tidyhandler" "ob_start" "ob_list_handlers" "ob_implicit_flush"
    "ob_iconv_handler" "ob_gzhandler" "ob_get_status" "ob_get_level"
    "ob_get_length" "ob_get_flush" "ob_get_contents" "ob_get_clean"
    "ob_flush" "ob_end_flush" "ob_end_clean" "ob_clean" "number_format"
    "nsapi_virtual" "nsapi_response_headers" "nsapi_request_headers"
    "notes_version" "notes_unread" "notes_search" "notes_nav_create"
    "notes_mark_unread" "notes_mark_read" "notes_list_msgs"
    "notes_header_info" "notes_find_note" "notes_drop_db"
    "notes_create_note" "notes_create_db" "notes_copy_db" "notes_body"
    "nl2br" "nl_langinfo" "ngettext" "next" "ncurses_wvline"
    "ncurses_wstandout" "ncurses_wstandend" "ncurses_wrefresh"
    "ncurses_wnoutrefresh" "ncurses_wmove" "ncurses_wmouse_trafo"
    "ncurses_whline" "ncurses_wgetch" "ncurses_werase"
    "ncurses_wcolor_set" "ncurses_wclear" "ncurses_wborder"
    "ncurses_wattrset" "ncurses_wattron" "ncurses_wattroff"
    "ncurses_waddstr" "ncurses_waddch" "ncurses_vline" "ncurses_vidattr"
    "ncurses_use_extended_names" "ncurses_use_env"
    "ncurses_use_default_colors" "ncurses_update_panels"
    "ncurses_ungetmouse" "ncurses_ungetch" "ncurses_typeahead"
    "ncurses_top_panel" "ncurses_timeout" "ncurses_termname"
    "ncurses_termattrs" "ncurses_start_color" "ncurses_standout"
    "ncurses_standend" "ncurses_slk_touch" "ncurses_slk_set"
    "ncurses_slk_restore" "ncurses_slk_refresh" "ncurses_slk_noutrefresh"
    "ncurses_slk_init" "ncurses_slk_color" "ncurses_slk_clear"
    "ncurses_slk_attrset" "ncurses_slk_attron" "ncurses_slk_attroff"
    "ncurses_slk_attr" "ncurses_show_panel" "ncurses_scrl"
    "ncurses_scr_set" "ncurses_scr_restore" "ncurses_scr_init"
    "ncurses_scr_dump" "ncurses_savetty" "ncurses_resetty"
    "ncurses_reset_shell_mode" "ncurses_reset_prog_mode"
    "ncurses_replace_panel" "ncurses_refresh" "ncurses_raw"
    "ncurses_qiflush" "ncurses_putp" "ncurses_prefresh"
    "ncurses_pnoutrefresh" "ncurses_panel_window" "ncurses_panel_below"
    "ncurses_panel_above" "ncurses_pair_content" "ncurses_noraw"
    "ncurses_noqiflush" "ncurses_nonl" "ncurses_noecho" "ncurses_nocbreak"
    "ncurses_nl" "ncurses_newwin" "ncurses_newpad" "ncurses_new_panel"
    "ncurses_napms" "ncurses_mvwaddstr" "ncurses_mvvline" "ncurses_mvinch"
    "ncurses_mvhline" "ncurses_mvgetch" "ncurses_mvdelch" "ncurses_mvcur"
    "ncurses_mvaddstr" "ncurses_mvaddnstr" "ncurses_mvaddchstr"
    "ncurses_mvaddchnstr" "ncurses_mvaddch" "ncurses_move_panel"
    "ncurses_move" "ncurses_mousemask" "ncurses_mouseinterval"
    "ncurses_mouse_trafo" "ncurses_meta" "ncurses_longname"
    "ncurses_killchar" "ncurses_keypad" "ncurses_keyok" "ncurses_isendwin"
    "ncurses_instr" "ncurses_insstr" "ncurses_insertln" "ncurses_insdelln"
    "ncurses_insch" "ncurses_init_pair" "ncurses_init_color"
    "ncurses_init" "ncurses_inch" "ncurses_hline" "ncurses_hide_panel"
    "ncurses_has_key" "ncurses_has_il" "ncurses_has_ic"
    "ncurses_has_colors" "ncurses_halfdelay" "ncurses_getyx"
    "ncurses_getmouse" "ncurses_getmaxyx" "ncurses_getch"
    "ncurses_flushinp" "ncurses_flash" "ncurses_filter"
    "ncurses_erasechar" "ncurses_erase" "ncurses_end" "ncurses_echochar"
    "ncurses_echo" "ncurses_doupdate" "ncurses_delwin" "ncurses_deleteln"
    "ncurses_delch" "ncurses_delay_output" "ncurses_del_panel"
    "ncurses_define_key" "ncurses_def_shell_mode" "ncurses_def_prog_mode"
    "ncurses_curs_set" "ncurses_color_set" "ncurses_color_content"
    "ncurses_clrtoeol" "ncurses_clrtobot" "ncurses_clear" "ncurses_cbreak"
    "ncurses_can_change_color" "ncurses_bottom_panel" "ncurses_border"
    "ncurses_bkgdset" "ncurses_bkgd" "ncurses_beep" "ncurses_baudrate"
    "ncurses_attrset" "ncurses_attron" "ncurses_attroff"
    "ncurses_assume_default_colors" "ncurses_addstr" "ncurses_addnstr"
    "ncurses_addchstr" "ncurses_addchnstr" "ncurses_addch" "natsort"
    "natcasesort" "mysqli_warning_count" "mysqli_use_result"
    "mysqli_thread_safe" "mysqli_thread_id" "mysqli_store_result"
    "mysqli_stmt_store_result" "mysqli_stmt_sqlstate"
    "mysqli_stmt_send_long_data" "mysqli_stmt_result_metadata"
    "mysqli_stmt_reset" "mysqli_stmt_prepare" "mysqli_stmt_param_count"
    "mysqli_stmt_num_rows" "mysqli_stmt_init" "mysqli_stmt_free_result"
    "mysqli_stmt_fetch" "mysqli_stmt_execute" "mysqli_stmt_error"
    "mysqli_stmt_errno" "mysqli_stmt_data_seek" "mysqli_stmt_close"
    "mysqli_stmt_bind_result" "mysqli_stmt_bind_param"
    "mysqli_stmt_affected_rows" "mysqli_stat" "mysqli_ssl_set"
    "mysqli_sqlstate" "mysqli_set_opt" "mysqli_server_init"
    "mysqli_server_end" "mysqli_send_query" "mysqli_send_long_data"
    "mysqli_select_db" "mysqli_rpl_query_type" "mysqli_rpl_probe"
    "mysqli_rpl_parse_enabled" "mysqli_rollback" "mysqli_report"
    "mysqli_real_query" "mysqli_real_escape_string" "mysqli_real_connect"
    "mysqli_query" "mysqli_prepare" "mysqli_ping" "mysqli_param_count"
    "mysqli_options" "mysqli_num_rows" "mysqli_num_fields"
    "mysqli_next_result" "mysqli_multi_query" "mysqli_more_results"
    "mysqli_master_query" "mysqli_kill" "mysqli_insert_id" "mysqli_init"
    "mysqli_info" "mysqli_get_server_version" "mysqli_get_server_info"
    "mysqli_get_proto_info" "mysqli_get_metadata" "mysqli_get_host_info"
    "mysqli_get_client_version" "mysqli_get_client_info"
    "mysqli_free_result" "mysqli_field_tell" "mysqli_field_seek"
    "mysqli_field_count" "mysqli_fetch_row" "mysqli_fetch_object"
    "mysqli_fetch_lengths" "mysqli_fetch_fields"
    "mysqli_fetch_field_direct" "mysqli_fetch_field" "mysqli_fetch_assoc"
    "mysqli_fetch_array" "mysqli_fetch" "mysqli_execute"
    "mysqli_escape_string" "mysqli_error" "mysqli_errno"
    "mysqli_enable_rpl_parse" "mysqli_enable_reads_from_master"
    "mysqli_embedded_connect" "mysqli_dump_debug_info"
    "mysqli_disable_rpl_parse" "mysqli_disable_reads_from_master"
    "mysqli_debug" "mysqli_data_seek" "mysqli_connect_error"
    "mysqli_connect_errno" "mysqli_connect" "mysqli_commit" "mysqli_close"
    "mysqli_client_encoding" "mysqli_character_set_name"
    "mysqli_change_user" "mysqli_bind_result" "mysqli_bind_param"
    "mysqli_autocommit" "mysqli_affected_rows" "mysql_unbuffered_query"
    "mysql_thread_id" "mysql_tablename" "mysql_stat" "mysql_select_db"
    "mysql_result" "mysql_real_escape_string" "mysql_query" "mysql_ping"
    "mysql_pconnect" "mysql_num_rows" "mysql_num_fields"
    "mysql_list_tables" "mysql_list_processes" "mysql_list_fields"
    "mysql_list_dbs" "mysql_insert_id" "mysql_info"
    "mysql_get_server_info" "mysql_get_proto_info" "mysql_get_host_info"
    "mysql_get_client_info" "mysql_free_result" "mysql_field_type"
    "mysql_field_table" "mysql_field_seek" "mysql_field_name"
    "mysql_field_len" "mysql_field_flags" "mysql_fetch_row"
    "mysql_fetch_object" "mysql_fetch_lengths" "mysql_fetch_field"
    "mysql_fetch_assoc" "mysql_fetch_array" "mysql_escape_string"
    "mysql_error" "mysql_errno" "mysql_drop_db" "mysql_db_query"
    "mysql_db_name" "mysql_data_seek" "mysql_create_db" "mysql_connect"
    "mysql_close" "mysql_client_encoding" "mysql_change_user"
    "mysql_affected_rows" "muscat_setup_net" "muscat_setup" "muscat_give"
    "muscat_get" "muscat_close" "mt_srand" "mt_rand" "mt_getrandmax"
    "mssql_select_db" "mssql_rows_affected" "mssql_result" "mssql_query"
    "mssql_pconnect" "mssql_num_rows" "mssql_num_fields"
    "mssql_next_result" "mssql_min_message_severity"
    "mssql_min_error_severity" "mssql_init" "mssql_guid_string"
    "mssql_get_last_message" "mssql_free_statement" "mssql_free_result"
    "mssql_field_type" "mssql_field_seek" "mssql_field_name"
    "mssql_field_length" "mssql_fetch_row" "mssql_fetch_object"
    "mssql_fetch_field" "mssql_fetch_batch" "mssql_fetch_assoc"
    "mssql_fetch_array" "mssql_execute" "mssql_data_seek" "mssql_connect"
    "mssql_close" "mssql_bind" "msql_tablename" "msql_select_db"
    "msql_result" "msql_regcase" "msql_query" "msql_pconnect"
    "msql_numrows" "msql_numfields" "msql_num_rows" "msql_num_fields"
    "msql_list_tables" "msql_list_fields" "msql_list_dbs"
    "msql_free_result" "msql_fieldtype" "msql_fieldtable" "msql_fieldname"
    "msql_fieldlen" "msql_fieldflags" "msql_field_type" "msql_field_table"
    "msql_field_seek" "msql_field_name" "msql_field_len"
    "msql_field_flags" "msql_fetch_row" "msql_fetch_object"
    "msql_fetch_field" "msql_fetch_array" "msql_error" "msql_drop_db"
    "msql_dbname" "msql_db_query" "msql_data_seek" "msql_createdb"
    "msql_create_db" "msql_connect" "msql_close" "msql_affected_rows"
    "msql" "msg_stat_queue" "msg_set_queue" "msg_send" "msg_remove_queue"
    "msg_receive" "msg_get_queue" "msession_unlock" "msession_uniq"
    "msession_timeout" "msession_set_data" "msession_set_array"
    "msession_set" "msession_randstr" "msession_plugin" "msession_lock"
    "msession_listvar" "msession_list" "msession_inc" "msession_get_data"
    "msession_get_array" "msession_get" "msession_find"
    "msession_disconnect" "msession_destroy" "msession_create"
    "msession_count" "msession_connect" "move_uploaded_file"
    "money_format" "mktime" "mkdir" "ming_useswfversion" "ming_setscale"
    "ming_setcubicthreshold" "min" "mime_content_type" "microtime"
    "mhash_keygen_s2k" "mhash_get_hash_name" "mhash_get_block_size"
    "mhash_count" "mhash" "method_exists" "metaphone" "memory_get_usage"
    "memcache_set" "memcache_replace" "memcache_pconnect"
    "memcache_increment" "memcache_getversion" "memcache_getstats"
    "memcache_get" "memcache_flush" "memcache_delete" "memcache_decrement"
    "memcache_debug" "memcache_connect" "memcache_close" "memcache_add"
    "mdecrypt_generic" "md5_file" "md5" "mcve_void" "mcve_verifysslcert"
    "mcve_verifyconnection" "mcve_uwait" "mcve_ub" "mcve_transsend"
    "mcve_transparam" "mcve_transnew" "mcve_transinqueue"
    "mcve_transactiontext" "mcve_transactionssent" "mcve_transactionitem"
    "mcve_transactionid" "mcve_transactioncv" "mcve_transactionbatch"
    "mcve_transactionavs" "mcve_transactionauth" "mcve_text_cv"
    "mcve_text_code" "mcve_text_avs" "mcve_settle" "mcve_settimeout"
    "mcve_setssl_files" "mcve_setssl" "mcve_setip" "mcve_setdropfile"
    "mcve_setblocking" "mcve_sale" "mcve_returnstatus" "mcve_returncode"
    "mcve_return" "mcve_responseparam" "mcve_qc" "mcve_preauthcompletion"
    "mcve_preauth" "mcve_ping" "mcve_parsecommadelimited" "mcve_override"
    "mcve_numrows" "mcve_numcolumns" "mcve_monitor" "mcve_maxconntimeout"
    "mcve_listusers" "mcve_liststats" "mcve_iscommadelimited"
    "mcve_initusersetup" "mcve_initengine" "mcve_initconn" "mcve_gut"
    "mcve_gl" "mcve_gft" "mcve_getuserparam" "mcve_getuserarg"
    "mcve_getheader" "mcve_getcommadelimited" "mcve_getcellbynum"
    "mcve_getcell" "mcve_force" "mcve_enableuser" "mcve_edituser"
    "mcve_disableuser" "mcve_destroyengine" "mcve_destroyconn"
    "mcve_deluser" "mcve_deleteusersetup" "mcve_deletetrans"
    "mcve_deleteresponse" "mcve_connectionerror" "mcve_connect"
    "mcve_completeauthorizations" "mcve_chngpwd" "mcve_chkpwd"
    "mcve_checkstatus" "mcve_bt" "mcve_adduserarg" "mcve_adduser"
    "mcrypt_ofb" "mcrypt_module_self_test" "mcrypt_module_open"
    "mcrypt_module_is_block_mode" "mcrypt_module_is_block_algorithm_mode"
    "mcrypt_module_is_block_algorithm"
    "mcrypt_module_get_supported_key_sizes"
    "mcrypt_module_get_algo_key_size" "mcrypt_module_get_algo_block_size"
    "mcrypt_module_close" "mcrypt_list_modes" "mcrypt_list_algorithms"
    "mcrypt_get_key_size" "mcrypt_get_iv_size" "mcrypt_get_cipher_name"
    "mcrypt_get_block_size" "mcrypt_generic_init" "mcrypt_generic_end"
    "mcrypt_generic_deinit" "mcrypt_generic" "mcrypt_encrypt"
    "mcrypt_enc_self_test" "mcrypt_enc_is_block_mode"
    "mcrypt_enc_is_block_algorithm_mode" "mcrypt_enc_is_block_algorithm"
    "mcrypt_enc_get_supported_key_sizes" "mcrypt_enc_get_modes_name"
    "mcrypt_enc_get_key_size" "mcrypt_enc_get_iv_size"
    "mcrypt_enc_get_block_size" "mcrypt_enc_get_algorithms_name"
    "mcrypt_ecb" "mcrypt_decrypt" "mcrypt_create_iv" "mcrypt_cfb"
    "mcrypt_cbc" "mcal_week_of_year" "mcal_time_valid" "mcal_store_event"
    "mcal_snooze" "mcal_reopen" "mcal_rename_calendar" "mcal_popen"
    "mcal_open" "mcal_next_recurrence" "mcal_list_events"
    "mcal_list_alarms" "mcal_is_leap_year" "mcal_fetch_event"
    "mcal_fetch_current_stream_event" "mcal_expunge"
    "mcal_event_set_title" "mcal_event_set_start"
    "mcal_event_set_recur_yearly" "mcal_event_set_recur_weekly"
    "mcal_event_set_recur_none" "mcal_event_set_recur_monthly_wday"
    "mcal_event_set_recur_monthly_mday" "mcal_event_set_recur_daily"
    "mcal_event_set_end" "mcal_event_set_description"
    "mcal_event_set_class" "mcal_event_set_category"
    "mcal_event_set_alarm" "mcal_event_init" "mcal_event_add_attribute"
    "mcal_delete_event" "mcal_delete_calendar" "mcal_days_in_month"
    "mcal_day_of_year" "mcal_day_of_week" "mcal_date_valid"
    "mcal_date_compare" "mcal_create_calendar" "mcal_close"
    "mcal_append_event" "mb_substr_count" "mb_substr"
    "mb_substitute_character" "mb_strwidth" "mb_strtoupper"
    "mb_strtolower" "mb_strrpos" "mb_strpos" "mb_strlen" "mb_strimwidth"
    "mb_strcut" "mb_split" "mb_send_mail" "mb_regex_set_options"
    "mb_regex_encoding" "mb_preferred_mime_name" "mb_parse_str"
    "mb_output_handler" "mb_list_encodings" "mb_language"
    "mb_internal_encoding" "mb_http_output" "mb_http_input" "mb_get_info"
    "mb_eregi_replace" "mb_eregi" "mb_ereg_search_setpos"
    "mb_ereg_search_regs" "mb_ereg_search_pos" "mb_ereg_search_init"
    "mb_ereg_search_getregs" "mb_ereg_search_getpos" "mb_ereg_search"
    "mb_ereg_replace" "mb_ereg_match" "mb_ereg" "mb_encode_numericentity"
    "mb_encode_mimeheader" "mb_detect_order" "mb_detect_encoding"
    "mb_decode_numericentity" "mb_decode_mimeheader"
    "mb_convert_variables" "mb_convert_kana" "mb_convert_encoding"
    "mb_convert_case" "max" "main" "mailparse_uudecode_all"
    "mailparse_stream_encode" "mailparse_rfc822_parse_addresses"
    "mailparse_msg_parse_file" "mailparse_msg_parse"
    "mailparse_msg_get_structure" "mailparse_msg_get_part_data"
    "mailparse_msg_get_part" "mailparse_msg_free"
    "mailparse_msg_extract_part_file" "mailparse_msg_extract_part"
    "mailparse_msg_create" "mailparse_determine_best_xfer_encoding" "mail"
    "lzf_optimized_for" "lzf_decompress" "lzf_compress" "ltrim" "lstat"
    "long2ip" "log1p" "log10" "log" "localtime" "localeconv" "list"
    "linkinfo" "link" "limititerator_valid" "limititerator_seek"
    "limititerator_rewind" "limititerator_next"
    "limititerator_getposition" "levenshtein" "ldap_unbind"
    "ldap_t61_to_8859" "ldap_start_tls" "ldap_sort" "ldap_set_rebind_proc"
    "ldap_set_option" "ldap_search" "ldap_sasl_bind" "ldap_rename"
    "ldap_read" "ldap_parse_result" "ldap_parse_reference"
    "ldap_next_reference" "ldap_next_entry" "ldap_next_attribute"
    "ldap_modify" "ldap_mod_replace" "ldap_mod_del" "ldap_mod_add"
    "ldap_list" "ldap_get_values_len" "ldap_get_values" "ldap_get_option"
    "ldap_get_entries" "ldap_get_dn" "ldap_get_attributes"
    "ldap_free_result" "ldap_first_reference" "ldap_first_entry"
    "ldap_first_attribute" "ldap_explode_dn" "ldap_error" "ldap_errno"
    "ldap_err2str" "ldap_dn2ufn" "ldap_delete" "ldap_count_entries"
    "ldap_connect" "ldap_compare" "ldap_close" "ldap_bind" "ldap_add"
    "ldap_8859_to_t61" "lcg_value" "ksort" "krsort" "key" "juliantojd"
    "jpeg2wbmp" "join" "jewishtojd" "jdtounix" "jdtojulian" "jdtojewish"
    "jdtogregorian" "jdtofrench" "jdmonthname" "jddayofweek"
    "java_last_exception_get" "java_last_exception_clear" "isset"
    "is_writeable" "is_writable" "is_uploaded_file" "is_subclass_of"
    "is_string" "is_soap_fault" "is_scalar" "is_resource" "is_real"
    "is_readable" "is_object" "is_numeric" "is_null" "is_nan" "is_long"
    "is_link" "is_integer" "is_int" "is_infinite" "is_float" "is_finite"
    "is_file" "is_executable" "is_double" "is_dir" "is_callable" "is_bool"
    "is_array" "is_a" "ircg_whois" "ircg_who" "ircg_topic"
    "ircg_set_on_die" "ircg_set_file" "ircg_set_current"
    "ircg_register_format_messages" "ircg_pconnect" "ircg_part"
    "ircg_oper" "ircg_notice" "ircg_nickname_unescape"
    "ircg_nickname_escape" "ircg_nick" "ircg_names" "ircg_msg"
    "ircg_lusers" "ircg_lookup_format_messages" "ircg_list" "ircg_kick"
    "ircg_join" "ircg_is_conn_alive" "ircg_invite" "ircg_ignore_del"
    "ircg_ignore_add" "ircg_html_encode" "ircg_get_username"
    "ircg_fetch_error_msg" "ircg_eval_ecmascript_params" "ircg_disconnect"
    "ircg_channel_mode" "iptcparse" "iptcembed" "ip2long" "intval"
    "ini_set" "ini_restore" "ini_get_all" "ini_get" "ini_alter"
    "ingres_rollback" "ingres_query" "ingres_pconnect" "ingres_num_rows"
    "ingres_num_fields" "ingres_field_type" "ingres_field_scale"
    "ingres_field_precision" "ingres_field_nullable" "ingres_field_name"
    "ingres_field_length" "ingres_fetch_row" "ingres_fetch_object"
    "ingres_fetch_array" "ingres_connect" "ingres_commit" "ingres_close"
    "ingres_autocommit" "inet_pton" "inet_ntop" "include_once" "include"
    "in_array" "import_request_variables" "implode" "imap_utf8"
    "imap_utf7_encode" "imap_utf7_decode" "imap_unsubscribe"
    "imap_undelete" "imap_uid" "imap_timeout" "imap_thread"
    "imap_subscribe" "imap_status" "imap_sort" "imap_setflag_full"
    "imap_setacl" "imap_set_quota" "imap_search" "imap_scanmailbox"
    "imap_rfc822_write_address" "imap_rfc822_parse_headers"
    "imap_rfc822_parse_adrlist" "imap_reopen" "imap_renamemailbox"
    "imap_qprint" "imap_ping" "imap_open" "imap_num_recent" "imap_num_msg"
    "imap_msgno" "imap_mime_header_decode" "imap_mailboxmsginfo"
    "imap_mail_move" "imap_mail_copy" "imap_mail_compose" "imap_mail"
    "imap_lsub" "imap_listsubscribed" "imap_listscan" "imap_listmailbox"
    "imap_list" "imap_last_error" "imap_headers" "imap_headerinfo"
    "imap_header" "imap_getsubscribed" "imap_getmailboxes" "imap_getacl"
    "imap_get_quotaroot" "imap_get_quota" "imap_fetchstructure"
    "imap_fetchheader" "imap_fetchbody" "imap_fetch_overview"
    "imap_expunge" "imap_errors" "imap_deletemailbox" "imap_delete"
    "imap_createmailbox" "imap_close" "imap_clearflag_full" "imap_check"
    "imap_bodystruct" "imap_body" "imap_binary" "imap_base64"
    "imap_append" "imap_alerts" "imap_8bit" "imagexbm" "imagewbmp"
    "imagetypes" "imagettftext" "imagettfbbox" "imagetruecolortopalette"
    "imagesy" "imagesx" "imagestringup" "imagestring" "imagesettile"
    "imagesetthickness" "imagesetstyle" "imagesetpixel" "imagesetbrush"
    "imagesavealpha" "imagerotate" "imagerectangle" "imagepstext"
    "imagepsslantfont" "imagepsloadfont" "imagepsfreefont"
    "imagepsextendfont" "imagepsencodefont" "imagepscopyfont"
    "imagepsbbox" "imagepolygon" "imagepng" "imagepalettecopy"
    "imageloadfont" "imageline" "imagelayereffect" "imagejpeg"
    "imageistruecolor" "imageinterlace" "imagegif" "imagegd2" "imagegd"
    "imagegammacorrect" "imagefttext" "imageftbbox" "imagefontwidth"
    "imagefontheight" "imagefilter" "imagefilltoborder"
    "imagefilledrectangle" "imagefilledpolygon" "imagefilledellipse"
    "imagefilledarc" "imagefill" "imageellipse" "imagedestroy"
    "imagedashedline" "imagecreatetruecolor" "imagecreatefromxpm"
    "imagecreatefromxbm" "imagecreatefromwbmp" "imagecreatefromstring"
    "imagecreatefrompng" "imagecreatefromjpeg" "imagecreatefromgif"
    "imagecreatefromgd2part" "imagecreatefromgd2" "imagecreatefromgd"
    "imagecreate" "imagecopyresized" "imagecopyresampled"
    "imagecopymergegray" "imagecopymerge" "imagecopy"
    "imagecolortransparent" "imagecolorstotal" "imagecolorsforindex"
    "imagecolorset" "imagecolorresolvealpha" "imagecolorresolve"
    "imagecolormatch" "imagecolorexactalpha" "imagecolorexact"
    "imagecolordeallocate" "imagecolorclosesthwb" "imagecolorclosestalpha"
    "imagecolorclosest" "imagecolorat" "imagecolorallocatealpha"
    "imagecolorallocate" "imagecharup" "imagechar" "imagearc"
    "imageantialias" "imagealphablending" "image2wbmp"
    "image_type_to_mime_type" "image_type_to_extension"
    "ignore_user_abort" "ifxus_write_slob" "ifxus_tell_slob"
    "ifxus_seek_slob" "ifxus_read_slob" "ifxus_open_slob"
    "ifxus_free_slob" "ifxus_create_slob" "ifxus_close_slob"
    "ifx_update_char" "ifx_update_blob" "ifx_textasvarchar" "ifx_query"
    "ifx_prepare" "ifx_pconnect" "ifx_num_rows" "ifx_num_fields"
    "ifx_nullformat" "ifx_htmltbl_result" "ifx_getsqlca" "ifx_get_char"
    "ifx_get_blob" "ifx_free_result" "ifx_free_char" "ifx_free_blob"
    "ifx_fieldtypes" "ifx_fieldproperties" "ifx_fetch_row" "ifx_errormsg"
    "ifx_error" "ifx_do" "ifx_create_char" "ifx_create_blob"
    "ifx_copy_blob" "ifx_connect" "ifx_close" "ifx_byteasvarchar"
    "ifx_blobinfile_mode" "ifx_affected_rows" "idate" "id3_set_tag"
    "id3_remove_tag" "id3_get_version" "id3_get_tag" "id3_get_genre_name"
    "id3_get_genre_list" "id3_get_genre_id" "iconv_substr" "iconv_strrpos"
    "iconv_strpos" "iconv_strlen" "iconv_set_encoding" "iconv_mime_encode"
    "iconv_mime_decode_headers" "iconv_mime_decode" "iconv_get_encoding"
    "iconv" "ibase_wait_event" "ibase_trans" "ibase_timefmt"
    "ibase_set_event_handler" "ibase_service_detach"
    "ibase_service_attach" "ibase_server_info" "ibase_rollback_ret"
    "ibase_rollback" "ibase_restore" "ibase_query" "ibase_prepare"
    "ibase_pconnect" "ibase_param_info" "ibase_num_params"
    "ibase_num_fields" "ibase_name_result" "ibase_modify_user"
    "ibase_maintain_db" "ibase_gen_id" "ibase_free_result"
    "ibase_free_query" "ibase_free_event_handler" "ibase_field_info"
    "ibase_fetch_row" "ibase_fetch_object" "ibase_fetch_assoc"
    "ibase_execute" "ibase_errmsg" "ibase_errcode" "ibase_drop_db"
    "ibase_delete_user" "ibase_db_info" "ibase_connect" "ibase_commit_ret"
    "ibase_commit" "ibase_close" "ibase_blob_open" "ibase_blob_info"
    "ibase_blob_import" "ibase_blob_get" "ibase_blob_echo"
    "ibase_blob_create" "ibase_blob_close" "ibase_blob_cancel"
    "ibase_blob_add" "ibase_backup" "ibase_affected_rows" "ibase_add_user"
    "hypot" "hwapi_userlist" "hwapi_user" "hwapi_unlock" "hwapi_srcsofdst"
    "hwapi_srcanchors" "hwapi_setcommitedversion" "hwapi_replace"
    "hwapi_remove" "hwapi_reason_type" "hwapi_reason_description"
    "hwapi_parents" "hwapi_objectbyanchor" "hwapi_object_value"
    "hwapi_object_title" "hwapi_object_remove" "hwapi_object_new"
    "hwapi_object_insert" "hwapi_object_count" "hwapi_object_attreditable"
    "hwapi_object_assign" "hwapi_object" "hwapi_new_content" "hwapi_move"
    "hwapi_lock" "hwapi_link" "hwapi_insertdocument"
    "hwapi_insertcollection" "hwapi_insertanchor" "hwapi_insert"
    "hwapi_info" "hwapi_identify" "hwapi_hwstat" "hwapi_hgcsp"
    "hwapi_ftstat" "hwapi_find" "hwapi_error_reason" "hwapi_error_count"
    "hwapi_dstofsrcanchors" "hwapi_dstanchors" "hwapi_dcstat"
    "hwapi_dbstat" "hwapi_copy" "hwapi_content_read"
    "hwapi_content_mimetype" "hwapi_content" "hwapi_children"
    "hwapi_checkout" "hwapi_checkin" "hwapi_attribute_values"
    "hwapi_attribute_value" "hwapi_attribute_langdepvalue"
    "hwapi_attribute_key" "hwapi_attribute" "hw_who" "hw_unlock" "hw_stat"
    "hw_setlinkroot" "hw_root" "hw_pipedocument" "hw_pconnect"
    "hw_output_document" "hw_objrec2array" "hw_new_document" "hw_mv"
    "hw_modifyobject" "hw_mapid" "hw_insertobject" "hw_insertdocument"
    "hw_insertanchors" "hw_insdoc" "hw_inscoll" "hw_info"
    "hw_incollections" "hw_identify" "hw_getusername" "hw_gettext"
    "hw_getsrcbydestobj" "hw_getremotechildren" "hw_getremote"
    "hw_getrellink" "hw_getparentsobj" "hw_getparents"
    "hw_getobjectbyqueryobj" "hw_getobjectbyquerycollobj"
    "hw_getobjectbyquerycoll" "hw_getobjectbyquery" "hw_getobject"
    "hw_getchilddoccollobj" "hw_getchilddoccoll" "hw_getchildcollobj"
    "hw_getchildcoll" "hw_getandlock" "hw_getanchorsobj" "hw_getanchors"
    "hw_free_document" "hw_errormsg" "hw_error" "hw_edittext" "hw_dummy"
    "hw_document_size" "hw_document_setcontent" "hw_document_content"
    "hw_document_bodytag" "hw_document_attributes" "hw_docbyanchorobj"
    "hw_docbyanchor" "hw_deleteobject" "hw_cp" "hw_connection_info"
    "hw_connect" "hw_close" "hw_childrenobj" "hw_children"
    "hw_changeobject" "hw_array2objrec" "http_build_query"
    "htmlspecialchars" "htmlentities" "html_entity_decode"
    "highlight_string" "highlight_file" "hexdec" "hebrevc" "hebrev"
    "headers_sent" "headers_list" "header" "gzwrite" "gzuncompress"
    "gztell" "gzseek" "gzrewind" "gzread" "gzputs" "gzpassthru" "gzopen"
    "gzinflate" "gzgetss" "gzgets" "gzgetc" "gzfile" "gzeof" "gzencode"
    "gzdeflate" "gzcompress" "gzclose" "gregoriantojd" "gmstrftime"
    "gmp_xor" "gmp_sub" "gmp_strval" "gmp_sqrtrem" "gmp_sqrt" "gmp_sign"
    "gmp_setbit" "gmp_scan1" "gmp_scan0" "gmp_random" "gmp_prob_prime"
    "gmp_powm" "gmp_pow" "gmp_popcount" "gmp_perfect_square" "gmp_or"
    "gmp_neg" "gmp_mul" "gmp_mod" "gmp_legendre" "gmp_jacobi" "gmp_invert"
    "gmp_intval" "gmp_init" "gmp_hamdist" "gmp_gcdext" "gmp_gcd"
    "gmp_fact" "gmp_divexact" "gmp_div_r" "gmp_div_qr" "gmp_div_q"
    "gmp_div" "gmp_com" "gmp_cmp" "gmp_clrbit" "gmp_and" "gmp_add"
    "gmp_abs" "gmmktime" "gmdate" "glob" "gettype" "gettimeofday"
    "gettext" "getservbyport" "getservbyname" "getrusage" "getrandmax"
    "getprotobynumber" "getprotobyname" "getopt" "getmyuid" "getmypid"
    "getmyinode" "getmygid" "getmxrr" "getlastmod" "getimagesize"
    "gethostbynamel" "gethostbyname" "gethostbyaddr" "getenv" "getdate"
    "getcwd" "getallheaders" "get_resource_type" "get_required_files"
    "get_parent_class" "get_object_vars" "get_meta_tags"
    "get_magic_quotes_runtime" "get_magic_quotes_gpc"
    "get_loaded_extensions" "get_included_files" "get_include_path"
    "get_html_translation_table" "get_headers" "get_extension_funcs"
    "get_defined_vars" "get_defined_functions" "get_defined_constants"
    "get_declared_interfaces" "get_declared_classes" "get_current_user"
    "get_class_vars" "get_class_methods" "get_class" "get_cfg_var"
    "get_browser" "gd_info" "fwrite" "functions.variable_functions"
    "functions.returning_values" "functions.internal"
    "functions.arguments" "function_exists" "func_num_args"
    "func_get_args" "func_get_arg" "ftruncate" "ftp_systype"
    "ftp_ssl_connect" "ftp_size" "ftp_site" "ftp_set_option" "ftp_rmdir"
    "ftp_rename" "ftp_rawlist" "ftp_raw" "ftp_quit" "ftp_pwd" "ftp_put"
    "ftp_pasv" "ftp_nlist" "ftp_nb_put" "ftp_nb_get" "ftp_nb_fput"
    "ftp_nb_fget" "ftp_nb_continue" "ftp_mkdir" "ftp_mdtm" "ftp_login"
    "ftp_get_option" "ftp_get" "ftp_fput" "ftp_fget" "ftp_exec"
    "ftp_delete" "ftp_connect" "ftp_close" "ftp_chmod" "ftp_chdir"
    "ftp_cdup" "ftp_alloc" "ftok" "ftell" "fstat" "fsockopen" "fseek"
    "fscanf" "fribidi_log2vis" "frenchtojd" "fread" "fputs" "fprintf"
    "fpassthru" "fopen" "fnmatch" "fmod" "flush" "floor" "flock"
    "floatval" "filteriterator_valid" "filteriterator_rewind"
    "filteriterator_next" "filteriterator_key"
    "filteriterator_getinneriterator" "filteriterator_current" "filetype"
    "filesize" "filepro_rowcount" "filepro_retrieve" "filepro_fieldwidth"
    "filepro_fieldtype" "filepro_fieldname" "filepro_fieldcount" "filepro"
    "fileperms" "fileowner" "filemtime" "fileinode" "filegroup"
    "filectime" "fileatime" "file_put_contents" "file_get_contents"
    "file_exists" "file" "fgetss" "fgets" "fgetcsv" "fgetc" "fflush"
    "feof" "fdf_set_version" "fdf_set_value" "fdf_set_target_frame"
    "fdf_set_submit_form_action" "fdf_set_status" "fdf_set_opt"
    "fdf_set_on_import_javascript" "fdf_set_javascript_action"
    "fdf_set_flags" "fdf_set_file" "fdf_set_encoding" "fdf_set_ap"
    "fdf_save_string" "fdf_save" "fdf_remove_item" "fdf_open_string"
    "fdf_open" "fdf_next_field_name" "fdf_header" "fdf_get_version"
    "fdf_get_value" "fdf_get_status" "fdf_get_opt" "fdf_get_flags"
    "fdf_get_file" "fdf_get_encoding" "fdf_get_attachment" "fdf_get_ap"
    "fdf_error" "fdf_errno" "fdf_enum_values" "fdf_create" "fdf_close"
    "fdf_add_template" "fdf_add_doc_javascript" "fclose" "fbsql_warnings"
    "fbsql_username" "fbsql_tablename" "fbsql_stop_db" "fbsql_start_db"
    "fbsql_set_transaction" "fbsql_set_password" "fbsql_set_lob_mode"
    "fbsql_select_db" "fbsql_rollback" "fbsql_result" "fbsql_read_clob"
    "fbsql_read_blob" "fbsql_query" "fbsql_pconnect" "fbsql_password"
    "fbsql_num_rows" "fbsql_num_fields" "fbsql_next_result"
    "fbsql_list_tables" "fbsql_list_fields" "fbsql_list_dbs"
    "fbsql_insert_id" "fbsql_hostname" "fbsql_get_autostart_info"
    "fbsql_free_result" "fbsql_field_type" "fbsql_field_table"
    "fbsql_field_seek" "fbsql_field_name" "fbsql_field_len"
    "fbsql_field_flags" "fbsql_fetch_row" "fbsql_fetch_object"
    "fbsql_fetch_lengths" "fbsql_fetch_field" "fbsql_fetch_assoc"
    "fbsql_fetch_array" "fbsql_error" "fbsql_errno" "fbsql_drop_db"
    "fbsql_db_status" "fbsql_db_query" "fbsql_database_password"
    "fbsql_database" "fbsql_data_seek" "fbsql_create_db"
    "fbsql_create_clob" "fbsql_create_blob" "fbsql_connect" "fbsql_commit"
    "fbsql_close" "fbsql_clob_size" "fbsql_change_user" "fbsql_blob_size"
    "fbsql_autocommit" "fbsql_affected_rows" "fam_suspend_monitor"
    "fam_resume_monitor" "fam_pending" "fam_open" "fam_next_event"
    "fam_monitor_file" "fam_monitor_directory" "fam_monitor_collection"
    "fam_close" "fam_cancel_monitor" "ezmlm_hash" "extract"
    "extension_loaded" "expm1" "explode" "exp" "exit" "exif_thumbnail"
    "exif_tagname" "exif_read_data" "exif_imagetype" "exec" "eval"
    "escapeshellcmd" "escapeshellarg" "error_reporting" "error_log"
    "eregi_replace" "eregi" "ereg_replace" "ereg" "end" "empty" "echo"
    "ebcdic2ascii" "easter_days" "easter_date" "each" "doubleval"
    "dotnet_load" "domxsltstylesheet_result_dump_mem"
    "domxsltstylesheet_result_dump_file" "domxsltstylesheet_process"
    "domxml_xslt_stylesheet_file" "domxml_xslt_stylesheet_doc"
    "domxml_xslt_stylesheet" "domxml_xmltree" "domxml_version"
    "domxml_open_mem" "domxml_open_file" "domxml_new_doc"
    "domprocessinginstruction_target" "domprocessinginstruction_data"
    "domnode_unlink_node" "domnode_set_namespace" "domnode_set_name"
    "domnode_set_content" "domnode_replace_node" "domnode_replace_child"
    "domnode_remove_child" "domnode_previous_sibling" "domnode_prefix"
    "domnode_parent_node" "domnode_owner_document" "domnode_node_value"
    "domnode_node_type" "domnode_node_name" "domnode_next_sibling"
    "domnode_last_child" "domnode_is_blank_node" "domnode_insert_before"
    "domnode_has_child_nodes" "domnode_has_attributes"
    "domnode_get_content" "domnode_first_child" "domnode_dump_node"
    "domnode_clone_node" "domnode_child_nodes" "domnode_attributes"
    "domnode_append_sibling" "domnode_append_child"
    "domnode_add_namespace" "domelement_tagname"
    "domelement_set_attribute" "domelement_remove_attribute"
    "domelement_has_attribute" "domelement_get_elements_by_tagname"
    "domelement_get_attribute_node" "domelement_get_attribute"
    "domdocumenttype_system_id" "domdocumenttype_public_id"
    "domdocumenttype_notations" "domdocumenttype_name"
    "domdocumenttype_internal_subset" "domdocumenttype_entities"
    "domdocument_xinclude" "domdocument_html_dump_mem"
    "domdocument_get_elements_by_tagname" "domdocument_get_element_by_id"
    "domdocument_dump_mem" "domdocument_dump_file"
    "domdocument_document_element" "domdocument_doctype"
    "domdocument_create_text_node"
    "domdocument_create_processing_instruction"
    "domdocument_create_entity_reference" "domdocument_create_element_ns"
    "domdocument_create_element" "domdocument_create_comment"
    "domdocument_create_cdata_section" "domdocument_create_attribute"
    "domdocument_add_root" "domattribute_value" "domattribute_specified"
    "domattribute_name" "dom_import_simplexml"
    "dom_domxpath_registernamespace" "dom_domxpath_query"
    "dom_domtext_splittext" "dom_domtext_iswhitespaceinelementcontent"
    "dom_domnodelist_item" "dom_domnode_replacechild"
    "dom_domnode_removechild" "dom_domnode_normalize"
    "dom_domnode_lookupprefix" "dom_domnode_lookupnamespaceuri"
    "dom_domnode_issupported" "dom_domnode_issamenode"
    "dom_domnode_insertbefore" "dom_domnode_haschildnodes"
    "dom_domnode_hasattributes" "dom_domnode_clonenode"
    "dom_domnode_appendchild" "dom_domnamednodemap_item"
    "dom_domnamednodemap_getnameditemns"
    "dom_domnamednodemap_getnameditem" "dom_domimplementation_hasfeature"
    "dom_domimplementation_createdocumenttype"
    "dom_domimplementation_createdocument" "dom_domelement_setattributens"
    "dom_domelement_setattributenodens" "dom_domelement_setattributenode"
    "dom_domelement_setattribute" "dom_domelement_removeattributens"
    "dom_domelement_removeattributenode" "dom_domelement_removeattribute"
    "dom_domelement_hasattributens" "dom_domelement_hasattribute"
    "dom_domelement_getelementsbytagnamens"
    "dom_domelement_getelementsbytagname" "dom_domelement_getattributens"
    "dom_domelement_getattributenodens" "dom_domelement_getattributenode"
    "dom_domelement_getattribute" "dom_domdocument_xinclude"
    "dom_domdocument_validate" "dom_domdocument_schemavalidatesource"
    "dom_domdocument_schemavalidate" "dom_domdocument_savexml"
    "dom_domdocument_savehtmlfile" "dom_domdocument_savehtml"
    "dom_domdocument_save" "dom_domdocument_relaxngvalidatesource"
    "dom_domdocument_relaxngvalidate" "dom_domdocument_normalize"
    "dom_domdocument_loadxml" "dom_domdocument_loadhtmlfile"
    "dom_domdocument_loadhtml" "dom_domdocument_load"
    "dom_domdocument_importnode" "dom_domdocument_getelementsbytagnamens"
    "dom_domdocument_getelementsbytagname"
    "dom_domdocument_getelementbyid" "dom_domdocument_createtextnode"
    "dom_domdocument_createprocessinginstruction"
    "dom_domdocument_createentityreference"
    "dom_domdocument_createelementns" "dom_domdocument_createelement"
    "dom_domdocument_createdocumentfragment"
    "dom_domdocument_createcomment" "dom_domdocument_createcdatasection"
    "dom_domdocument_createattributens" "dom_domdocument_createattribute"
    "dom_domcharacterdata_substringdata"
    "dom_domcharacterdata_replacedata" "dom_domcharacterdata_insertdata"
    "dom_domcharacterdata_deletedata" "dom_domcharacterdata_appenddata"
    "dom_domattr_isid" "dns_get_record" "dns_get_mx" "dns_check_record"
    "dngettext" "dl" "diskfreespace" "disk_total_space" "disk_free_space"
    "dirname" "directoryiterator_valid" "directoryiterator_rewind"
    "directoryiterator_next" "directoryiterator_key"
    "directoryiterator_iswritable" "directoryiterator_isreadable"
    "directoryiterator_islink" "directoryiterator_isfile"
    "directoryiterator_isexecutable" "directoryiterator_isdot"
    "directoryiterator_isdir" "directoryiterator_getpathname"
    "directoryiterator_getpath" "directoryiterator_getfilename"
    "directoryiterator_getchildren" "directoryiterator_filetype"
    "directoryiterator_filesize" "directoryiterator_fileperms"
    "directoryiterator_fileowner" "directoryiterator_filemtime"
    "directoryiterator_fileinode" "directoryiterator_filegroup"
    "directoryiterator_filectime" "directoryiterator_fileatime"
    "directoryiterator_current" "directoryiterator_construct" "dio_write"
    "dio_truncate" "dio_tcsetattr" "dio_stat" "dio_seek" "dio_read"
    "dio_open" "dio_fcntl" "dio_close" "die" "dgettext" "delete" "deg2rad"
    "defined" "define_syslog_variables" "define" "decoct" "dechex"
    "decbin" "debugger_on" "debugger_off" "debug_print_backtrace"
    "debug_backtrace" "deaggregate" "dcngettext" "dcgettext" "dbx_sort"
    "dbx_query" "dbx_fetch_row" "dbx_escape_string" "dbx_error"
    "dbx_connect" "dbx_compare" "dbx_close" "dbplus_xunlockrel"
    "dbplus_xlockrel" "dbplus_update" "dbplus_unselect" "dbplus_unlockrel"
    "dbplus_undoprepare" "dbplus_undo" "dbplus_tremove" "dbplus_tcl"
    "dbplus_sql" "dbplus_setindexbynumber" "dbplus_setindex"
    "dbplus_savepos" "dbplus_rzap" "dbplus_runlink" "dbplus_rsecindex"
    "dbplus_rrename" "dbplus_rquery" "dbplus_ropen" "dbplus_rkeys"
    "dbplus_restorepos" "dbplus_resolve" "dbplus_rcrtlike"
    "dbplus_rcrtexact" "dbplus_rcreate" "dbplus_rchperm" "dbplus_prev"
    "dbplus_open" "dbplus_next" "dbplus_lockrel" "dbplus_last"
    "dbplus_info" "dbplus_getunique" "dbplus_getlock" "dbplus_freerlocks"
    "dbplus_freelock" "dbplus_freealllocks" "dbplus_flush" "dbplus_first"
    "dbplus_find" "dbplus_errno" "dbplus_errcode" "dbplus_curr"
    "dbplus_close" "dbplus_chdir" "dbplus_aql" "dbplus_add" "dbmreplace"
    "dbmopen" "dbmnextkey" "dbminsert" "dbmfirstkey" "dbmfetch"
    "dbmexists" "dbmdelete" "dbmclose" "dblist" "dbase_replace_record"
    "dbase_pack" "dbase_open" "dbase_numrecords" "dbase_numfields"
    "dbase_get_record_with_names" "dbase_get_record"
    "dbase_get_header_info" "dbase_delete_record" "dbase_create"
    "dbase_close" "dbase_add_record" "dba_sync" "dba_replace" "dba_popen"
    "dba_optimize" "dba_open" "dba_nextkey" "dba_list" "dba_key_split"
    "dba_insert" "dba_handlers" "dba_firstkey" "dba_fetch" "dba_exists"
    "dba_delete" "dba_close" "date" "cyrus_unbind" "cyrus_query"
    "cyrus_connect" "cyrus_close" "cyrus_bind" "cyrus_authenticate"
    "cybercash_encr" "cybercash_decr" "cybercash_base64_encode"
    "cybercash_base64_decode" "current" "curl_version" "curl_setopt"
    "curl_multi_select" "curl_multi_remove_handle" "curl_multi_init"
    "curl_multi_info_read" "curl_multi_getcontent" "curl_multi_exec"
    "curl_multi_close" "curl_multi_add_handle" "curl_init" "curl_getinfo"
    "curl_exec" "curl_error" "curl_errno" "curl_copy_handle" "curl_close"
    "ctype_xdigit" "ctype_upper" "ctype_space" "ctype_punct" "ctype_print"
    "ctype_lower" "ctype_graph" "ctype_digit" "ctype_cntrl" "ctype_alpha"
    "ctype_alnum" "crypt" "create_function" "crc32" "crack_opendict"
    "crack_getlastmessage" "crack_closedict" "crack_check"
    "cpdf_translate" "cpdf_text" "cpdf_stroke" "cpdf_stringwidth"
    "cpdf_show_xy" "cpdf_show" "cpdf_setrgbcolor_stroke"
    "cpdf_setrgbcolor_fill" "cpdf_setrgbcolor" "cpdf_setmiterlimit"
    "cpdf_setlinewidth" "cpdf_setlinejoin" "cpdf_setlinecap"
    "cpdf_setgray_stroke" "cpdf_setgray_fill" "cpdf_setgray"
    "cpdf_setflat" "cpdf_setdash" "cpdf_set_word_spacing"
    "cpdf_set_viewer_preferences" "cpdf_set_title" "cpdf_set_text_rise"
    "cpdf_set_text_rendering" "cpdf_set_text_pos" "cpdf_set_text_matrix"
    "cpdf_set_subject" "cpdf_set_page_animation" "cpdf_set_leading"
    "cpdf_set_keywords" "cpdf_set_horiz_scaling" "cpdf_set_font_map_file"
    "cpdf_set_font_directories" "cpdf_set_font" "cpdf_set_current_page"
    "cpdf_set_creator" "cpdf_set_char_spacing" "cpdf_set_action_url"
    "cpdf_scale" "cpdf_save_to_file" "cpdf_save" "cpdf_rotate_text"
    "cpdf_rotate" "cpdf_rmoveto" "cpdf_rlineto" "cpdf_restore" "cpdf_rect"
    "cpdf_place_inline_image" "cpdf_page_init" "cpdf_output_buffer"
    "cpdf_open" "cpdf_newpath" "cpdf_moveto" "cpdf_lineto"
    "cpdf_import_jpeg" "cpdf_global_set_document_limits"
    "cpdf_finalize_page" "cpdf_finalize" "cpdf_fill_stroke" "cpdf_fill"
    "cpdf_end_text" "cpdf_curveto" "cpdf_continue_text"
    "cpdf_closepath_stroke" "cpdf_closepath_fill_stroke" "cpdf_closepath"
    "cpdf_close" "cpdf_clip" "cpdf_circle" "cpdf_begin_text" "cpdf_arc"
    "cpdf_add_outline" "cpdf_add_annotation" "count_chars" "count" "cosh"
    "cos" "copy" "convert_uuencode" "convert_uudecode"
    "convert_cyr_string" "constant" "connection_timeout"
    "connection_status" "connection_aborted" "compact" "com_set"
    "com_release" "com_propset" "com_propput" "com_propget"
    "com_print_typeinfo" "com_message_pump" "com_load_typelib" "com_load"
    "com_isenum" "com_invoke" "com_get_active_object" "com_get"
    "com_event_sink" "com_create_guid" "com_addref" "closelog" "closedir"
    "clearstatcache" "classkit_method_rename" "classkit_method_remove"
    "classkit_method_redefine" "classkit_method_copy"
    "classkit_method_add" "class_exists" "chunk_split" "chroot" "chr"
    "chown" "chop" "chmod" "chgrp" "checkdnsrr" "checkdate" "chdir" "ceil"
    "ccvs_void" "ccvs_textvalue" "ccvs_status" "ccvs_sale" "ccvs_reverse"
    "ccvs_return" "ccvs_report" "ccvs_new" "ccvs_lookup" "ccvs_init"
    "ccvs_done" "ccvs_delete" "ccvs_count" "ccvs_command" "ccvs_auth"
    "ccvs_add" "call_user_method_array" "call_user_method"
    "call_user_func_array" "call_user_func" "cal_to_jd" "cal_info"
    "cal_from_jd" "cal_days_in_month"
    "cachingrecursiveiterator_haschildren"
    "cachingrecursiveiterator_getchildren" "cachingiterator_valid"
    "cachingiterator_tostring" "cachingiterator_rewind"
    "cachingiterator_next" "cachingiterator_hasnext" "bzwrite" "bzread"
    "bzopen" "bzflush" "bzerrstr" "bzerror" "bzerrno" "bzdecompress"
    "bzcompress" "bzclose" "bindtextdomain" "bindec"
    "bind_textdomain_codeset" "bin2hex" "bcsub" "bcsqrt" "bcscale"
    "bcpowmod" "bcpow" "bcmul" "bcmod" "bcdiv" "bccomp" "bcadd" "basename"
    "base64_encode" "base64_decode" "base_convert" "atanh" "atan2" "atan"
    "assert_options" "assert" "aspell_suggest" "aspell_new"
    "aspell_check_raw" "aspell_check" "asort" "asinh" "asin"
    "ascii2ebcdic" "arsort" "arrayobject_offsetunset"
    "arrayobject_offsetset" "arrayobject_offsetget"
    "arrayobject_offsetexists" "arrayobject_getiterator"
    "arrayobject_count" "arrayobject_construct" "arrayobject_append"
    "arrayiterator_valid" "arrayiterator_seek" "arrayiterator_rewind"
    "arrayiterator_next" "arrayiterator_key" "arrayiterator_current"
    "array_walk_recursive" "array_walk" "array_values" "array_unshift"
    "array_unique" "array_uintersect_uassoc" "array_uintersect_assoc"
    "array_uintersect" "array_udiff_uassoc" "array_udiff_assoc"
    "array_udiff" "array_sum" "array_splice" "array_slice" "array_shift"
    "array_search" "array_reverse" "array_reduce" "array_rand"
    "array_push" "array_pop" "array_pad" "array_multisort"
    "array_merge_recursive" "array_merge" "array_map" "array_keys"
    "array_key_exists" "array_intersect_ukey" "array_intersect_uassoc"
    "array_intersect_key" "array_intersect_assoc" "array_intersect"
    "array_flip" "array_filter" "array_fill" "array_diff_ukey"
    "array_diff_uassoc" "array_diff_key" "array_diff_assoc" "array_diff"
    "array_count_values" "array_combine" "array_chunk"
    "array_change_key_case" "array" "apache_setenv"
    "apache_response_headers" "apache_request_headers" "apache_note"
    "apache_lookup_uri" "apache_getenv" "apache_get_version"
    "apache_get_modules" "apache_child_terminate" "aggregation_info"
    "aggregate_properties_by_regexp" "aggregate_properties_by_list"
    "aggregate_properties" "aggregate_methods_by_regexp"
    "aggregate_methods_by_list" "aggregate_methods" "aggregate_info"
    "aggregate" "addslashes" "addcslashes" "acosh" "acos" "abs")
  "PHP functions.")

(defconst php-types
  '("array" "boolean" "bool" "char" "const" "double" "float"
    "integer" "int" "long" "mixed" "object" "real"
    "string")
  "PHP types.")

(defconst php-superglobals
  '("_GET" "_POST" "_COOKIE" "_SESSION" "_ENV" "GLOBALS"
    "_SERVER" "_FILES" "_REQUEST")
  "PHP superglobal variables.")

(defvar php-functions-alist
  (mapcar
   (lambda(word)
     (cons word nil))
   php-functions)
  "An association list of the PHP functions for completion use.")

(defvar php-keywords-alist
  (mapcar
   (lambda(word)
     (cons word nil))
   php-keywords)
  "An association list of the PHP keywords for completion use.")

(defvar php-constants-alist
  (mapcar
   (lambda(word)
     (cons word nil))
   php-constants)
  "An association list of the PHP constants for completion use.")

(defvar php-types-alist
  (mapcar
   (lambda(word)
     (cons word nil))
   php-types)
  "An association list of the PHP types for completion use.")

(defvar php-superglobals-alist
          (mapcar
           (lambda(word)
             (cons (concat word "[]")  nil))
           php-superglobals)
  "An association list of the PHP superglobals for completion use.")

;;; internal
(defun php-list-to-regexp(start wordlist end)
  "Takes a list and returns the regexp \\(elem1\\|elem2\\|...\\)"
  (let ((regexp start))
    (mapcar (lambda(elem)
          (setq regexp (concat regexp elem "\\|")))
        wordlist)
    (concat (substring regexp 0 -2) ; cutting the last "\\|"
        end)
    ))

(defconst php-identifier
  (eval-when-compile
    '"[a-zA-Z\_\x7f-\xff][a-zA-Z0-9\_\x7f-\xff]*")
  "Characters in a PHP identifier.")

;; Set up font locking
(defconst php-font-lock-keywords-1
  (list
   ;; Fontify constants
   `(,(php-list-to-regexp "\\<\\(" php-constants "\\)\\>") (1 font-lock-constant-face))
   ;; Fontify keywords
   `(,(php-list-to-regexp "\\<\\(" php-keywords "\\)\\>") (1 font-lock-keyword-face))

   ;; Fontify keywords and targets, and case default tags.
   (list "\\<\\(break\\|case\\|continue\\)\\>[ \t]*\\(-?\\(?:\\sw\\|\\s_\\)+\\)?"
     '(1 font-lock-keyword-face) '(2 font-lock-constant-face t t))
   ;; This must come after the one for keywords and targets.
   '(":" ("^[ \t]*\\(\\(?:\\sw\\|\\s_\\)+\\)[ \t]*:[ \t]*$"
      (beginning-of-line) (end-of-line)
      (1 font-lock-constant-face)))

   ;; treat 'print' as keyword only when not used like a function name
   '("\\<print\\s-*(" . default)
   '("\\<print\\>" . font-lock-keyword-face)

   ;; Fontify PHP tag
   '("<\\?\\(php\\)?" . font-lock-constant-face)
   '("\\?>" . font-lock-constant-face)

   ;; Fontify ASP-style tag
   '("<\\%\\(=\\)?" . font-lock-constant-face)
   '("\\%>" . font-lock-constant-face)

   )
  "Subdued level highlighting for PHP mode.")

(defconst php-font-lock-keywords-2
  (append
   php-font-lock-keywords-1
   (list

    ;; class declaration
    '("\\<\\(class\\|interface\\)[ \t]*\\(\\(?:\\sw\\|\\s_\\)+\\)?"
      (1 font-lock-keyword-face) (2 font-lock-type-face nil t))
    ;; handle several words specially, to include following word,
    ;; thereby excluding it from unknown-symbol checks later
    ;; FIX to handle implementing multiple
    ;; currently breaks on "class Foo implements Bar, Baz"
    '("\\<\\(new\\|extends\\|implements\\)\\s-+\\$?\\(\\(?:\\sw\\|\\s_\\)+\\)"
      (1 font-lock-keyword-face) (2 font-lock-type-face))

    ;; function declaration
    '("\\<\\(function\\)\\s-+&?\\(\\(?:\\sw\\|\\s_\\)+\\)\\s-*("
      (1 font-lock-keyword-face)
      (2 font-lock-function-name-face nil t))

    ;; class hierarchy
    '("\\(self\\|parent\\)\\W" (1 font-lock-constant-face nil nil))

    ;; method and variable features
    '("\\<\\(private\\|protected\\|public\\)\\s-+\\$?\\(?:\\sw\\|\\s_\\)+"
      (1 font-lock-keyword-face))

    ;; method features
    '("^[ \t]*\\(abstract\\|static\\|final\\)\\s-+\\$?\\(?:\\sw\\|\\s_\\)+"
      (1 font-lock-keyword-face))

    ;; variable features
    '("^[ \t]*\\(static\\|const\\)\\s-+\\$?\\(?:\\sw\\|\\s_\\)+"
      (1 font-lock-keyword-face))
    ))
  "Medium level highlighting for PHP mode.")

(defconst php-font-lock-keywords-3
  (append
   php-font-lock-keywords-2
   (list

    ;; <word> or </word> for HTML
    '("</?\\sw+[^>]*>" . font-lock-constant-face)

    ;; HTML entities
    '("&\\w+;" . font-lock-variable-name-face)

    ;; warn about '$' immediately after ->
    '("\\$\\(?:\\sw\\|\\s_\\)+->\\s-*\\(\\$\\)\\(\\(?:\\sw\\|\\s_\\)+\\)"
      (1 font-lock-warning-face) (2 default))

    ;; warn about $word.word -- it could be a valid concatenation,
    ;; but without any spaces we'll assume $word->word was meant.
    '("\\$\\(?:\\sw\\|\\s_\\)+\\(\\.\\)\\sw"
      1 font-lock-warning-face)

    ;; Warn about ==> instead of =>
    '("==+>" . font-lock-warning-face)

   ;; Fontify types
   `(,(php-list-to-regexp "(\\s-*\\(" php-types "\\)\\s-*)")
     1 font-lock-type-face)
   ;; Fontify super globals
   `(,(php-list-to-regexp "\\$\\(" php-superglobals "\\)\\W")
     1 font-lock-constant-face nil nil)

    ;; PHP5: function declarations may contain classes as parameters type
    `(,(concat "[(,]\\s-*\\(\\(?:\\sw\\|\\s_\\)+\\)\\s-+\\$\\(?:\\sw\\|\\s_\\)+\\>")
      1 font-lock-type-face)

    ;; Fontify variables and function calls
    '("\\$\\(this\\|that\\)\\W" (1 font-lock-constant-face nil nil))
    '("\\$\\(\\(?:\\sw\\|\\s_\\)+\\)" (1 font-lock-variable-name-face)) ; $variable
    '("->\\(\\(?:\\sw\\|\\s_\\)+\\)" (1 font-lock-variable-name-face t t)) ; ->variable
    '("->\\(\\(?:\\sw\\|\\s_\\)+\\)\\s-*(" . (1 default t t)) ; ->function_call
    '("\\(?:\\sw\\|\\s_\\)+::\\(?:\\sw\\|\\s_\\)+\\s-*(" . default) ; class::method call
    '("\\<\\(?:\\sw\\|\\s_\\)+\\s-*[[(]" . default) ; word( or word[
    '("\\<[0-9]+" . default)        ; number (also matches word)

    ;; Warn on any words not already fontified
    '("\\<\\(?:\\sw\\|\\s_\\)+\\>" . font-lock-warning-face)
    ;; Fontify internal functions
    `(,(php-list-to-regexp  "\\<\\(" php-functions "\\)\\>")
      1 font-lock-function-name-face)

    ))
  "Gauchy level highlighting for PHP mode.")

(defconst php-font-lock-syntactic-keywords
  (if xemacsp nil
    ;; Mark shell-style comments.  font-lock handles this in a
    ;; separate pass from normal syntactic scanning (somehow), so we
    ;; get a chance to mark these in addition to C and C++ style
    ;; comments.  This only works in GNU Emacs, not XEmacs 21 which
    ;; seems to ignore this same code if we try to use it.
    (list
     ;; Mark _all_ # chars as being comment-start.  That will be
     ;; ignored when inside a quoted string.
     '("\\(\#\\)"
       (1 (11 . nil)))
     ;; Mark all newlines ending a line with # as being comment-end.
     ;; This causes a problem, premature end-of-comment, when '#'
     ;; appears inside a multiline C-style comment.  Oh well.
     '("#.*\\([\n]\\)"
       (1 (12 . nil)))
     )))


;;; Function name completion (this functions import from css-mode.el)
(defun php-complete-at-point()
  "If point is at the end of a property name: returns it."
  (let ((end (point))
    (start (+ (re-search-backward "[^-_A-Za-z]") 1)))
    (goto-char end)
    (buffer-substring start end)))

; internal
(defun php-maximum-common(alt1 alt2)
  "Returns the maximum common starting substring of alt1 and alt2."
  (let* ((maxlen (min (length alt1) (length alt2)))
     (alt1 (substring alt1 0 maxlen))
     (alt2 (substring alt2 0 maxlen)))
    (while (not (string= (substring alt1 0 maxlen)
             (substring alt2 0 maxlen)))
      (setq maxlen (- maxlen 1)))
    (substring alt1 0 maxlen)))

; internal
(defun php-common-beginning(alts)
  "Returns the maximum common starting substring of all alts elements."
  (let ((common (car alts)))
    (dolist (alt (cdr alts) common)
      (setq common (php-maximum-common alt common)))))

(defun php-complete-frame(completions)
  ; This code stolen from message.el. Kudos to larsi.
  (let ((cur (current-buffer)))
    (pop-to-buffer "*Completions*")
    (buffer-disable-undo (current-buffer))
    (let ((buffer-read-only nil))
      (erase-buffer)
      (let ((standard-output (current-buffer)))
    (display-completion-list (sort completions 'string<)))
      (goto-char (point-min))
      (pop-to-buffer cur))))

(defun php-complete(alist)
  "Complete the function name, keywords, constants, types
at the point from known PHP words."
  (interactive)
  (let* ((word   (php-complete-at-point))
     (alts   (all-completions word alist))
     (wordlen (length word)))
    (if (= (length alts) 1)
    (insert (substring (car alts) wordlen))
      (let ((beg (php-common-beginning alts)))
    (if (not (string= beg word))
        (insert (substring beg wordlen))
      (insert (substring
           (completing-read "Completion: " alist nil
                    nil word)
           wordlen)))))))

(defun php-complete-constant ()
  "Complete the constants name at the point from known PHP constants."
  (interactive)
  (php-complete php-constants-alist))

(defun php-complete-function ()
  "Complete the function name at the point from known PHP functions."
  (interactive)
  (php-complete php-functions-alist))

(defun php-complete-keyword ()
  "Complete the keyword at the point from known PHP keywords."
  (interactive)
  (php-complete php-keywords-alist))

(defun php-complete-superglobal ()
  "Complete the superglobal variable name at the point from known PHP superglobals."
  (interactive)
  (php-complete php-superglobals-alist))

(defun php-complete-type ()
  "Complete the types name at the point from known PHP types."
  (interactive)
  (php-complete php-types-alist))


;; Create "default" symbol for GNU Emacs so that both XEmacs and GNU
;; emacs can refer to the default face by a variable named "default".
(unless (boundp 'default)
  (defvar default 'default))

;; Create faces for XEmacs
(unless (boundp 'font-lock-keyword-face)
  (copy-face 'bold 'font-lock-keyword-face))
(unless (boundp 'font-lock-constant-face)
  (copy-face 'font-lock-keyword-face 'font-lock-constant-face))

(provide 'php-mode)

;;; php-mode.el ends here
